#ifndef _collection_h
#define _collection_h

#include <vector>
#include <string>
#ifndef WIN32
#include <plotter.h>
#include <unistd.h>
#endif
#include "common.h"

using namespace std;

class Site;
class Path;
class DTEDGrid;

class Collection
{
public:
	Collection();
	Collection(char *fname);
	~Collection();

	unsigned int SiteCount()                      { return (unsigned int)m_vSites.size(); }
	unsigned int PathCount()                      { return (unsigned int)m_vPaths.size(); }
	unsigned int TotalCoordCount();

	void CoordBounds(Site *extra, double *lonmin, double *latmin, 
					 double *lonmax, double *latmax, bool verbose);
	void CoordBounds(Collection *extra, double *lonmin, double *latmin, 
					 double *lonmax, double *latmax, bool verbose);
	void GraphicBounds(Site *extra, int &xmin, int &ymin, 
					   int &xmax, int &ymax);
	void GraphicBounds(Collection *extra, int &xmin, int &ymin, 
					   int &xmax, int &ymax);

	bool GeneratePath(float inc, bool interpalt);

	long MaxPathAltitude();
	long MinPathAltitude();
	long MaxSiteAltitude();
	long MinSiteAltitude();

	int AddSite(Site *v);
	void DelSite(int index);
	int FindSite(Site *v);
	void SetGrid(DTEDGrid *v)						{ m_pGrid = v; }
	void Clear();

	Site *Sites(int index);
	Path *Paths(int index);
	Site *GlobalSite(unsigned int index);

	bool LoadSitesFromFile(char *fname);
	bool LoadSitesFromFile(string fname)			{ return LoadSitesFromFile((char*)fname.c_str()); }
	bool SaveSitesToFile(char *fname);
	bool SaveSitesToFile(string fname)				{ return SaveSitesToFile((char*)fname.c_str()); }

#ifdef _INCLUDE_LOAD_SAVE_PATHS
	bool LoadPathsFromFile(char *fname);
	bool LoadPathsFromFile(string fname)			{ return LoadPathsFromFile((char*)fname.c_str()); }
	bool SavePathsToFile(char *fname);
	bool SavePathsToFile(string fname)				{ return SavePathsToFile((char*)fname.c_str()); }

	bool SavePathsForGnuplot(char *base_fname, Site *ref = NULL)
	{ return SavePathsForGnuplot(string(base_fname), ref); }
	bool SavePathsForGnuplot(string base_fname, Site *ref = NULL);
#endif

protected:
	string ExtractFilePath(char *fname);

	vector<Site*> m_vSites;
	vector<Path*> m_vPaths;
	DTEDGrid *m_pGrid;

private:
};

#endif
