#ifndef _contour_h
#define _contour_h

#include <cstdio>
#include <string>
#include <plot.h>

#define xsect(p1, p2) (h[p2]*xh[p1]-h[p1]*xh[p2])/(h[p2]-h[p1])
#define ysect(p1, p2) (h[p2]*yh[p1]-h[p1]*yh[p2])/(h[p2]-h[p1])
#define min(x, y)     (x<y?x:y)
#define max(x, y)     (x>y?x:y)

using namespace std;

typedef struct _tpoint
{
  int x;
  int y;
} TPoint;

class Contour
{
 public:
  typedef struct CallbackData
  {
    float x1;
    float y1;
    float x2;
    float y2;
    float z;
    Contour *thog;
  } CallbackData;
  typedef void (*DrawCallback)(CallbackData *data, void *param);
  
  Contour(string fname = "");
  ~Contour();

  bool Create(int width, int height, int xmin, int ymin);
  int Calculate(string &fname, DrawCallback cb, void *param);
  void Clear();

  float ScaleWidth() const           { return m_fScaleWidth; }
  float &ScaleWidth()                { return m_fScaleWidth; }
  float ScaleHeight() const          { return m_fScaleHeight; }
  float &ScaleHeight()               { return  m_fScaleHeight; }
  int &XMin()                        { return m_iXMin; }
  int &YMin()                        { return m_iYMin; }
  void ContourCount(int v)           { _setcount(v); }
  void Levels(int i, float v)
  {
    if (0 <= i && i < m_iContourCount)
      m_pfLevels[i] = v;
    else printf("Levels(%d, %f): out of bounds (max=%d)\n", i, v, m_iContourCount);
  }
  void XCoords(int i, float v)
  {
    if (0 <= i && i < m_iWidth)
      m_pfXCoords[i] = v;
    else printf("XCoords(%d, %f): out of bounds (max=%d)\n", i, v, m_iWidth);
  }
  void YCoords(int i, float v)
  {
    if (0 <= i && i < m_iHeight)
      m_pfYCoords[i] = v;
    else printf("YCoords(%d, %f): out of bounds (max=%d)\n", i, v, m_iHeight);
  }
  float Levels(int i)
  {
    if (0 <= i && i < m_iContourCount)
      return m_pfLevels[i];
    return -10000.0;
  }
  float XCoords(int i)
  {
    if (0 <= i && i < m_iWidth)
      return m_pfXCoords[i];
    return -10000.0;
  }
  float YCoords(int i)
  {
    if (0 <= i && i < m_iHeight)
      return m_pfYCoords[i];
    return -10000.0;
  }
  void ZData(int r, int c, float v);

  int Width() const                  { return m_iWidth; }
  int Height() const                 { return m_iHeight; }
  int ContourCount() const           { return m_iContourCount; }
  
 protected:
  void _Allocate(int width, int height);
  void _Deallocate(bool all = false);
  void _SaveSegment(CallbackData *data);
  int _calculate();
  void _setcount(int v);

 private:
  float m_fScaleWidth;
  float m_fScaleHeight;
  int m_iWidth;
  int m_iHeight;
  int m_iXMin;
  int m_iYMin;
  int m_iContourCount;
  float *m_pfLevels;
  float *m_pfXCoords;
  float *m_pfYCoords;
  float **m_ppfZData;
  FILE *m_pLogFile;
  string m_sFileName;
  DrawCallback m_pCallbackPtr;
  void *m_pCallbackParam;
};

#endif
