#ifndef _dtedcell_h
#define _dtedcell_h

#include <iostream>
#include <cstdio>
#include <string>
#include <vector>
#include <math.h>
#include <ctype.h>

#include "dtedstructs.h"

class Site;
class Coord;

using namespace std;

class DTED;
typedef void (*FloatCallback)(DTED *obj, float val);

class DTED
{
public:
	DTED(char *fname);
	DTED(string &fname);
	~DTED();

	bool Loaded()                                        { return m_bLoaded; }
	double OriginLatVal()                                { return m_Header.uhl.proc.origin_lat; }
	double OriginLonVal()                                { return m_Header.uhl.proc.origin_lon; }
	short AltitudeByPost(int x, int y);
	short AltitudeByGeodetic(double lat, double lon, bool interpolate, int radius = 0);
	short AltitudeBySite(Site *point);
	int LatitudeCount();
	int LongitudeCount();
	int MinAltitude();
	int MaxAltitude();
	int LatOffset()                                      { return m_iLatOffset; }
	void LatOffset(int v)                                { m_iLatOffset = v; }
	int LonOffset()                                      { return m_iLonOffset; }
	void LonOffset(int v)                                { m_iLonOffset = v; }
	short Level()                                        { return m_sLevel; }
	short Interval();
	void MapCoordToPost(double lat, double lon, int &x_index, int &y_index/*, bool interpolate*/);
	void MapPostToGraphic(int &x_index, int &y_index, int xmin, int ymin);
	Coord *MapPostToCoord(int x_index, int y_index);
	void MapCoordToGraphic(double lat, double lon, int &x_index, int &y_index);
	void CalcWindowSize(int &xsize, int &ysize, float &xrat, float &yrat);

	int PlotXMin()                                       { return m_iPlotXMin; }
	int PlotXMax()                                       { return m_iPlotXMax; }
	int PlotYMin()                                       { return m_iPlotYMin; }
	int PlotYMax()                                       { return m_iPlotYMax; }
	bool PlotFake()                                      { return m_bPlotFake; }

	static string FindFile(string dpath, string filespec);
	static void BuildFileList(double xmin, double ymin, int xcnt, int ycnt,
		string path, vector<string> &list);

	static int PointCount[3];
	static int GridSize[3];

	static string PlotOriginLat;
	static string PlotOriginLon;
	static int str2lat(const char *str, int len);
	static int str2lon(const char *str, int len);
	static string lat2str(int lat);
	static string lon2str(int lon);
	static FloatCallback LoadStatusCB;
	static FloatCallback GraphStatusCB;

protected:
	bool loadfromfile(const char *fname);
	bool readheader();
	int readband(dat_struct *dat);

	dted_header m_Header;
	dted_data_vector m_vData;
	FILE *m_pFilePtr;
	short m_sLevel;
	bool m_bBigEndian;
	bool m_bLoaded;
	int m_iLatOffset;
	int m_iLonOffset;
	int m_iMinAltitude;
	int m_iMaxAltitude;

	int m_iPlotXMin, m_iPlotYMin;
	int m_iPlotXMax, m_iPlotYMax;
	int m_iPlotZoomLevel;
	bool m_bPlotFake;

private:
	bool _endian();
};

typedef vector<DTED*> dted_vector;

#endif
