#ifndef _dtedgrid_h
#define _dtedgrid_h

/*
Author: James A. Rogers (RogersJA@navair.navy.mil)
Date:   2003-11-19 1133 PST (-8:00)
File:   dtedgrid.{cpp,h}

Provides a framework for a grid of DTED cells, in order to span
multiple cells.
*/

#include <string>
using namespace std;

class DTED;
class FastPath;
class Site;
class Coord;

class DTEDGrid
{
public:
    typedef void (*StringCallback)(void *Sender, const char *str);

	DTEDGrid(int width, int height, double origin_lat, double origin_lon);
	~DTEDGrid();

	void CalcWindowSize(int &xsize, int &ysize, float &xrat, float &yrat);

	bool AddCell(DTED *cell, int &x_index, int &y_index);
	bool DelCell(int x_index, int y_index);
	DTED *Cell(int x, int y);
	void Cell(DTED *cell, int x, int y);

	short AltitudeByPost(int x_index, int y_index);
	short AltitudeByGeodetic(double lat, double lon, bool interpolate);
	DTED *MapCoordToCell(double lat, double lon, int *x_index, int *y_index);
	DTED *MapGraphicToCellPtr(int *x_index, int *y_index);
	void MapCoordToGraphic(double lat, double lon, int *x_index, int *y_index);
	Coord *MapGraphicToCoord(int x_index, int y_index);
	int Level()                                  { return m_iLevel; }
	int LongitudeCount();
	int LatitudeCount();
	int MinAltitude();
	int MaxAltitude();
	int LatCountPerCell();
	int LonCountPerCell();

	int BoundWidth();
	int BoundHeight();

	FastPath *GeneratePath(Site *src, Site *dest, float inc, bool interpalt);

	bool Allocated()                             { return m_bAllocated; }
	double OriginLat()                           { return m_dOriginLat; }
	double OriginLon()                           { return m_dOriginLon; }
	void OriginLat(double v)                     { m_dOriginLat = v; }
	void OriginLon(double v)                     { m_dOriginLon = v; }
	int Width()                                  { return m_iWidth; }
	int Height()                                 { return m_iHeight; }

    void SetStatusCallback(StringCallback p, void *data);
	StringCallback GetStatusCallback(void **data);

	static DTEDGrid *gPtr;
	static DTEDGrid *Load(double bxmin, double bymin, double bxmax,
		double bymax, string path, bool verbose);
	static DTED *_dtedlookupfunc(double lat, double lon);

protected:
	DTED ***m_ppGrid;
	int m_iWidth;
	int m_iHeight;
	int m_iLevel;
	double m_dOriginLat;
	double m_dOriginLon;
	bool m_bAllocated;
	string m_sContourFileName;
    StringCallback m_pStatusCB;
    void *m_pStatusCB_Data;

private:
	bool _alloc();
	bool _dealloc();
};

#endif
