#ifndef _dted_h
#define _dted_h

#include <vector>

using namespace std;

#pragma pack(1)

#define WINDOW_SIZE              530
#define DEFAULT_COLOR_SCALE       10

#define UHL_OFFSET            0x0000
#define DSI_OFFSET            0x0050
#define ACC_OFFSET            0x02D8
#define DAT_OFFSET            0x0D64

#define UHL_LENGTH            0x0050
#define DSI_LENGTH            0x0288
#define ACC_LENGTH            0x0A8C

#define LVL1_COUNT              1201
#define LVL2_COUNT              3601

#define DAT_SENTINEL            0xAA

#define UHL_COUNT                 13
const int uhl_offsets[13] = { 0, 3, 4, 12, 20, 24, 28, 32, 35, 47, 51, 55, 56 };
const int uhl_lengths[13] = { 3, 1, 8,  8,  4,  4,  4,  3, 12,  4,  4,  1, 24 };

struct _uhl_struct_raw
{
	char hdr[3];                           // 'UHL'
	char unused;                           // 1
	char origin_lon[8];                    // DDDMMSSH
	char origin_lat[8];                    // DDDMMSSH
	char interval_lon[4];                  // SSSS, tenths of seconds
	char interval_lat[4];                  // SSSS, tenths of seconds
	char abs_vertacc[4];                   // m, right justified
	char security_code[3];                 // S - Secret
	// C - Confidential
	// U - Unclassified
	// R - Restricted
	char unique_ref[12];
	char bandcount_lon[4];                 // based on latitude && dted level
	char bandcount_lat[4];                 // LVL1_COUNT || LVL2_COUNT
	char multiple_acc;                     // 0 - Single, 1 - Multiple
	char reserved[24];
};

struct _uhl_struct_proc
{
	int origin_lon;
	int origin_lat;
	int interval_lon;
	int interval_lat;
	int abs_vertacc;
	char security_code[3];
	int bandcount_lon;
	int bandcount_lat;
	bool multiple_acc;
	short series;
};

typedef struct
{
	struct _uhl_struct_raw raw;
	struct _uhl_struct_proc proc;
} uhl_struct;



#define DSI_COUNT           42
const int dsi_offsets[42] = { 0,   3,   4,   6,   33,  59,  64,  79,  87,  89,  90,  94,  98, /*13*/
102, 110, 126, 135, 137, 141, 144, 149, 159, 163, 185, 194, 204,/*13*/
211, 219, 226, 234, 241, 249, 256, 264, 273, 277, 281, 285, 289,/*13*/
291, 392, 492 };                                                /* 3*/
const int dsi_lengths[42] = { 3,  1, 2,  27, 26, 5, 15, 8, 2, 1, 4, 4, 4, 8, 16, 9, 2, 4, 3, 5,
10, 4, 22, 9,  10, 7, 8,  7, 8, 7, 8, 7, 8, 9, 4,  4, 4, 4, 2, 101,
100, 156 };

struct _dsi_struct_raw
{
	char hdr[3];                           // 'DSI'
	char security_code;                    // S - Secret
	// C - Confidential
	// U - Unclassified
	// R - Restricted
	char secctrl_markings[2];
	char sechandling[27];
	char reserved0[26];
	char series_des[5];                    // DTED0, DTED1, DTED2
	char unique_req[15];
	char reserved1[8];
	char edition_no[2];
	char matmer_version;
	char maint_date[4];
	char matmer_date[4];
	char maint_desc[4];
	char producer_code[8];
	char reserved2[16];
	char product_spec[9];
	char revision_ctrl[2];
	char product_spec_date[4];
	char vertical_datum[3];                // MSL, E96
	char horiz_datum[5];                   // 'WGS84'
	char collection_system[10];
	char compile_date[4];
	char reserved3[22];
	char origin_lat[9];                    // DDDMMSS.SH
	char origin_lon[10];                   // DDDMMSS.SH
	char sw_lat[7];                        // DDMMSSH
	char sw_lon[8];                        // DDDMMSSH
	char nw_lat[7];
	char nw_lon[8];
	char ne_lat[7];
	char ne_lon[8];
	char se_lat[7];
	char se_lon[8];
	char orient_angle[9];                 // DDDMMSS.S 9 265 Clockwise angle wrt true North
	char interval_lat[4];                 // SSSS tenths of seconds
	char interval_lon[4];                 // SSSS tenths of seconds
	char bandcount_lat[4];                // LVL1_COUNT || LVL2_COUNT
	char bandcount_lon[4];                // LVL1_COUNT || LVL2_COUNT
	char partialcell_indic[2];            // 00 = complete, 01-99 = % coverage
	char reserved4[101];
	char reserved5[100];
	char reserved6[156];
};

struct _dsi_struct_proc
{
	char security_code;
	string series_des;
	int origin_lat;
	int origin_lon;
	float sw_lat;
	float sw_lon;
	float nw_lat;
	float nw_lon;
	float ne_lat;
	float ne_lon;
	float se_lat;
	float se_lon;
	float orient_angle;
	int interval_lat;
	int interval_lon;
	int bandcount_lat;
	int bandcount_lon;
	short partial;
};

typedef struct
{
	struct _dsi_struct_raw raw;
	struct _dsi_struct_proc proc;
} dsi_struct;



typedef struct _acc_coord_struct
{
	char latitude[9];                     // DDMMSS.SH
	char longitude[10];                   // DDDMMSS.SH
} acc_coord_struct;

typedef struct _acc_subregion_struct
{
	char abs_horiz_acc[4];
	char abs_vert_acc[4];
	char rel_horiz_acc[4];
	char rel_vert_acc[4];
	char coord_count[2];                  // 01-14 coordinates
	_acc_coord_struct coords[14];
} acc_subregion_struct;

typedef struct _acc_struct
{
	char hdr[3];                          // 'ACC'
	char abs_horiz_acc[4];
	char abs_vert_acc[4];
	char rel_horiz_acc[4];
	char rel_vert_acc[4];
	char reserved0[4];
	char reserved1;
	char reserved2[31];
	char subregions_exist[2];             // 00 = no subregions, 02-09 = sub#
	_acc_subregion_struct subregions[9];
	char reserved3[18];
	char reserved4[69];
} acc_struct;


typedef struct _dat_struct
{
	unsigned short level;
	unsigned short count;
	unsigned short valid;
	struct
	{
		unsigned char sentinel;            // 0xAA
		unsigned char block_count[3];      // sequential block count
		unsigned char lon_count[2];        // meridian count (true lon = lon_count*interval+origin)
		unsigned char lat_count[2];        // parallel count (true lat = lat_count*interval+origin)
	} hdr;
	short *data;
	unsigned char checksum[4];
} dat_struct;



typedef struct
{
	uhl_struct uhl;
	dsi_struct dsi;
	acc_struct acc;
} dted_header;


typedef vector<dat_struct> dted_data_vector;

#pragma pack()

#endif
