#ifndef _fastpath_h
#define _fastpath_h

/*
  Author: James A. Rogers (RogersJA@navair.navy.mil)
  Date:   2003-11-30 1533 PST (-8:00)
  File:   fastpath.{cpp,h}

  Encapsulates a series of coordinates describing a great circle path.
  When the path is being built, the altitudes of each coordinate
  are lookup up from a DTED database.  This is a faster, less
  memory-allocation intensive version of path.{cpp,h}.
*/

#include <sstream>
#ifdef _GC_DEBUG
#include <cstdio>
#endif

#ifdef WIN32
#define _USE_MATH_DEFINES
#include <math.h>
#endif

class Site;
class DTED;
class DTEDGrid;
class GreatCircle;

using namespace std;

class DTED;
typedef DTED *(*DTEDLookupCallback)(double lat, double lon);

class FastPath
{
 public:
  FastPath();
  ~FastPath();

  bool CreatePath1(Site *src, Site *dest, DTED *cell, DTEDLookupCallback callBack,
		   bool include_lastpos, float inc, bool interpalt);
  bool CreatePath2(Site *src, Site *dest, DTED *cell, DTEDLookupCallback cbDted,
		   bool include_lastpos, float inc, bool interpalt);
  bool SaveToFile(char *fname);

  int TravelLOS(Site *src, Site *dest, int &obsalt);
  float FindMaxAngle(Site *src, Site *dest, int &atidx, int &atalt);
  
  int Altitude(int index);
  float Distance(int index);
  Site *Sites(int index);
  float DeltaDistance()                              { return m_fDeltaDistance; }
  int Size()                                         { return m_iSize; }
  int Capacity()                                     { return m_iCapacity; }

  static bool Debug;
  
 protected:
  int *m_ipAltitude;
  float *m_fpDistance;
  Site **m_pSites;
  int m_iSize;
  int m_iCapacity;
  float m_fDeltaDistance;
#ifdef _GC_DEBUG
  FILE *m_pFilePtr;
#endif

 private:
  double _arccos(double x, double y);
  int _addpoint(int alt, float dist);
};

#endif
