#include "graph_win32.h"
#include "dted.h"
#include "dtedgrid.h"
#include "path.h"
#include "coord.h"
#include "site.h"
//#include "collection.h"

#include "plotter.h"

//#define _COLOR_CHANNEL_IS_USHORT

Graph::Graph(int type)
{
	m_iPlotterType = type;
	m_pHandle.p = NULL;
	m_bOpen = false;
	m_pCollection = NULL;
}

Graph::Graph(int type, DTED *d, Collection *sc)
{
	m_iPlotterType = type;
	m_pHandle.p = NULL;
	m_bOpen = false;
	m_pCollection = sc;

	Allocate(d);
}

Graph::Graph(int type, DTEDGrid *g, Collection *sc)
{
	m_iPlotterType = type;
	m_pHandle.p = NULL;
	m_bOpen = false;
	m_pCollection = sc;

	Allocate(g, -1, -1);
}

Graph::~Graph()
{
	if (m_pHandle.p != NULL && m_bOpen == true)
	{
		m_pHandle.p->closepl();
		delete m_pHandle.p;
		m_pHandle.p = NULL;
	}
}

Plotter *Graph::Handle()
{
	return m_pHandle.p;
}

void Graph::SetCollection(Collection *sc)
{
	m_pCollection = sc;
}

bool Graph::Allocate(DTED *d)
{
	int xsz = WINDOW_SIZE, ysz = WINDOW_SIZE;
	char buf[64];
	float xn, yn;
	int x_max = d->LongitudeCount();
	int y_max = d->LatitudeCount();

	d->CalcWindowSize(xsz, ysz, xn, yn);

	_snprintf(buf, 64-1, "%dx%d", xsz, ysz);
	Plotter::parampl("BITMAPSIZE", buf);

	switch (m_iPlotterType)
	{
	case gtX: 
		m_pHandle.x = new XPlotter(stdin);
		break;

	case gtPNG:
		m_pOutputFile = fopen("plot.png", "w+");
		m_pHandle.png = new PNGPlotter(m_pOutputFile);
		break;

	case gtBMP:
		m_pOutputFile = fopen("plot.bmp", "w+");
		m_pHandle.bmp = new BMPPlotter(m_pOutputFile);
		break;

	default:
		cout<<"Graph::Allocate(): Invalid plotter type specified: "<<m_iPlotterType<<endl;
		return false;
	}

	if (m_pHandle.p->openpl() < 0)
	{
		m_bOpen = false;
		return false;
	}
	m_bOpen = true;

	if (xn == 1.0 && yn == 1.0)
		m_pHandle.p->fspace (0.0f, 0.0f, (float)x_max, (float)y_max);
	else m_pHandle.p->fspace (0.0f, 0.0f, x_max*xn, y_max*yn);

	return true;
}

bool Graph::Allocate(DTEDGrid *g, int maxwidth, int maxheight)
{
#ifdef _NO_PLOTXY
	return true;
#else
	int xsz = WINDOW_SIZE, ysz = WINDOW_SIZE;
	char buf[64];
	float xn = 1.0, yn = 1.0;
	int x_max = g->BoundWidth();
	int y_max = g->BoundHeight();
	DTEDGrid::StringCallback p;
	void *d_ptr;
	char str[512];

	p = g->GetStatusCallback(&d_ptr);

	/* calculates necessary window size, including tight
	bounds and compensating for number size of grid */
	g->CalcWindowSize(xsz, ysz, xn, yn);

	printf("\txn = %f, yn = %f\n", xn, yn);
	if (xn == 1.0 && yn == 1.0)
		printf("using unscaled (%d, %d, %d, %d)\n", 0, 0, x_max, y_max);
	else
	{
		printf("using scaled (%d, %d, %f, %f)\n", 0, 0, x_max*xn, y_max*yn);
		x_max = (int)((float)x_max*xn);
		y_max = (int)((float)y_max*yn);
	}

	//xsz *= 2;
	//ysz *= 2;

	if (maxwidth != -1 && maxwidth < xsz)
	{
		float scale = 1.0f-((float)xsz/(float)maxwidth);
		printf("greater than maxwidth of %d; scaling by %f\n", maxwidth, scale);
		xsz = maxwidth;
		ysz -= (int)((float)ysz*fabs(scale));
	}
	if (maxheight != -1 && maxheight < ysz)
	{
		float scale = 1.0f-((float)ysz/(float)maxheight);
		printf("greater than maxheight of %d; scaling by %f\n", maxheight, scale);
		xsz -= (int)((float)xsz*fabs(scale));
		ysz = maxheight;
	}

	//xsz = maxwidth;
	//ysz = maxheight;
	_snprintf(buf, 64-1, "%dx%d", xsz, ysz);
	Plotter::parampl("BITMAPSIZE", buf);

	if (p && d_ptr)
	{
		_snprintf(str, 512, "using %dx%d for window size", xsz, ysz);
		p(d_ptr, str);
	}

	switch (m_iPlotterType)
	{
	case gtX: 
		m_pHandle.x = new XPlotter(stdin);
		break;

	case gtPNG:
		m_pOutputFile = fopen("plot.png", "w+");
		m_pHandle.png = new PNGPlotter(m_pOutputFile);
		break;

	case gtBMP:
		m_pOutputFile = fopen("plot.bmp", "w+");
		m_pHandle.bmp = new BMPPlotter(m_pOutputFile);
		break;

	default:
		cout<<"Graph::Allocate(): Invalid plotter type specified: "<<m_iPlotterType<<endl;
		return false;
	}

	if (m_pHandle.p->openpl() < 0)
	{
		m_bOpen = false;
		return false;
	}
	m_bOpen = true;

	m_pHandle.p->bgcolor(0xaaaa, 0xaaaa, 0xaaaa);

	ipoint2_t p1;
	ipoint2_t p2;

	p1.x = -1;
	p1.y = -1;
	p2.x = -1;
	p2.y = -1;

	//x_max = maxwidth;
	//y_max = maxheight;

	if (p1.x == -1)
		p1.x = 0;
	if (p1.y == -1)
		p1.y = 0;
	if (p2.x == -1)
		p2.x = x_max;
	if (p2.y == -1)
		p2.y = y_max;
	m_pHandle.p->space (p1.x, p1.y, p2.x, p2.y);

	if (p && d_ptr)
	{
		_snprintf(str, 512, "using mapping for grid: (%d, %d, %d, %d)", 
				p1.x, p1.y, p2.x, p2.y);
		p(d_ptr, str);
	}

	m_pHandle.p->erase();

	return true;
#endif
}

bool Graph::Generate(DTED *d)
{
	m_iMinAltitude = d->MinAltitude();
	m_iMaxAltitude = d->MaxAltitude();

	return drawcell(d, 0, 0, d->PlotXMin(), d->PlotYMin(), d->PlotXMax(), d->PlotYMax());
}

bool Graph::Generate(DTEDGrid *g)
{
#ifdef _NO_PLOTXY
	return true;
#else
	int x, y;
	ipoint2_t p1, p2;
	DTEDGrid::StringCallback p;
	void *d_ptr;
	char buf[512];

	p = g->GetStatusCallback(&d_ptr);

	p1.x = -1;
	p1.y = -1;
	p2.x = -1;
	p2.y = -1;

	//m_iMinAltitude = g->MinAltitude(p1.x, p1.y, p2.x, p2.y);
	//m_iMaxAltitude = g->MaxAltitude(p1.x, p1.y, p2.x, p2.y);
	m_iMinAltitude = g->MinAltitude();
	m_iMaxAltitude = g->MaxAltitude();

	if (p && d_ptr)
	{
		_snprintf(buf, 512, "Graph bounds: (%d, %d), (%d, %d)", p1.x, p1.y, p2.x, p2.y);
		p(d_ptr, buf);

		_snprintf(buf, 512, "Graph max altitude: %d", m_iMaxAltitude);
		p(d_ptr, buf);

		_snprintf(buf, 512, "Graph min altitude: %d", m_iMinAltitude);
		p(d_ptr, buf);
	}

	for (x = 0; x < g->Width(); x++)
	{
		for (y = 0; y < g->Height(); y++)
		{
			if (p && d_ptr)
			{
				_snprintf(buf, 512, "drawing cell %d/%d", (x*g->Height())+y+1, 
							g->Width()*g->Height());
				p(d_ptr, buf);
			}

			drawcell(g->Cell(x, y), x, y, p1.x, p1.y, p2.x, p2.y);
		}
	}

	return true;
#endif
}

bool Graph::DrawPath(DTED *d, Path *path, ushort col[3])
{
	if (d == NULL || path == NULL || m_pHandle.p == NULL || m_bOpen == false)
		return false;

#ifdef _NO_PLOTXY
	return true;
#else
	int i;
	Coord *p1;
	Coord *p2;

	m_pHandle.p->pencolor(col[0], col[1], col[2]);

	for (i = 0; i < (int)path->Length()-1; i++)
	{
		ipoint2_t ip1 = {0, 0}, ip2 = {0, 0};

		p1 = path->Location(i);
		p2 = path->Location(i+1);

		d->MapCoordToPost(p1->Latitude, p1->Longitude, ip1.x, ip1.y);
		d->MapCoordToPost(p2->Latitude, p2->Longitude, ip2.x, ip2.y);

		m_pHandle.p->line(ip1.x, ip1.y, ip2.x, ip2.y);
	}

	return true;
#endif
}

bool Graph::Close()
{
	if (m_pHandle.p == NULL)
		return false;

	m_pHandle.p->closepl();

	if (m_iPlotterType == gtPNG || m_iPlotterType == gtBMP)
		fclose(m_pOutputFile);

	m_bOpen = false;

	return true;
}

void Graph::LoadContours(string &fname, DTEDGrid *grid, bool tight_bounds)
{
	FILE *fp;

	struct _pos
	{
		float x1, y1;
		float x2, y2;
		float z;
	} p;
	struct _hdr
	{
		unsigned long magic;
		int width;
		int height;
	} h;

	if (m_pHandle.p == NULL)
		return;

	fp = fopen(fname.c_str(), "r");
	if (!fp)
		return;

	fread(&h, sizeof(struct _hdr), 1, fp);
	if (h.magic != 0xdeadbeef)
	{
		fclose(fp);
		return;
	}

	m_pHandle.p->pencolor(0xaaaa, 0xaaaa, 0xaaaa);
	while (!feof(fp) && fread(&p, sizeof(struct _pos), 1, fp) == 1)
	{
#ifndef _NO_PLOTXY
#ifdef _TIGHT_BOUNDS
		if (tight_bounds == true)
		{
			if (grid->PlotXMin() != -1 &&
				((p.x1 < grid->PlotXMin()) || (p.x2 < grid->PlotXMin())))
				continue;
			if (grid->PlotXMax() != -1 &&
				((p.x1 > grid->PlotXMax()) || (p.x2 > grid->PlotXMax())))
				continue;
			if (grid->PlotYMin() != -1 &&
				((p.y1 < grid->PlotYMin()) || (p.y1 < grid->PlotYMin())))
				continue;
			if (grid->PlotYMax() != -1 &&
				((p.y1 > grid->PlotYMax()) || (p.y1 > grid->PlotYMax())))
				continue;

			/*if (grid->PlotXMin() != -1)*/
			p.x1 = p.x1/*-grid->PlotXMin()*/, p.x2 = p.x2/*-grid->PlotXMin()*/;

			/*if (grid->PlotYMin() != -1)*/
			p.y1 = p.y1/*-grid->PlotYMin()*/, p.y2 = p.y2/*-grid->PlotYMin()*/;
		}
#endif
#endif

		m_pHandle.p->fline(p.x1, p.y1, p.x2, p.y2);
	}

	fclose(fp);
}

#define _GRAYRAMP
bool Graph::drawcell(DTED *cell, int xindex, int yindex, int xmin, int ymin, int xmax, int ymax)
{
	int x_last, y_last, x, y, px, py;
#ifndef _GRAYRAMP
	int diffalt;
#endif
	unsigned short color[3];
	long xoff, yoff;
	short pc;

	if (m_bOpen == false || m_pHandle.p == NULL)
		return false;

	x_last = cell->LongitudeCount();
	y_last = cell->LatitudeCount();

	xoff = xindex*x_last;
	yoff = yindex*y_last;

	if (cell->PlotFake() == true)
		return true;

	if (x_last != y_last || x_last == 0)
	{
		cout<<"graph::drawcell() error: dimensions unequal or zero\n";
		return false;
	}

	m_pHandle.p->pencolor(0xffff, 0, 0);

#ifndef _GRAYRAMP
	diffalt = m_iMaxAltitude-m_iMinAltitude;
#endif

	/*printf("graph::drawcell(): xindex=%d, yindex=%d, p1=(%d,%d), p2=(%d,%d)\n",
	xindex, yindex, xmin, ymin, xmax, ymax);*/

	for (x = 0; x < x_last; x++)
	{
		if (xmin != -1 && (xoff+x < xmin))
			continue;
		if (xmax != -1 && (xoff+x > xmax))
			break;

		for (y = 0; y < y_last; y++)
		{
			if (ymin != -1 && (yoff+y < ymin))
				continue;
			if (ymax != -1 && (yoff+y > ymax))
				break;

			/*if (xmin != -1)
			px = xoff+x-xmin;
			else*/ px = xoff+x;

			/*if (ymin != -1)
			py = yoff+y-ymin;
			else*/ py = yoff+y;

			pc = cell->AltitudeByPost(x, y);
#ifdef _GRAYRAMP
			grayramp(pc, m_iMinAltitude, m_iMaxAltitude, &color[0], &color[1], &color[2]);
#else
			colorramp(diffalt, pc-m_iMinAltitude, &color[0], &color[1], &color[2]);
#endif

			m_pHandle.p->pencolor(color[0], color[1], color[2]);
			m_pHandle.p->point(px, py);
		}
	}

	return true;
}

void Graph::colorramp(int range, int value, ushort *red, ushort *green, ushort *blue)
{
	bool greyscale = false;
	int pixel;

	if (range == 0)
		range = 1;

	/* this will need to be fixed for Death Valley */
#if 1
	if (value < 0)
		value = 0;
#endif

	/* this shouldn't happen... */
	if (value > range)
		value = range;

	pixel = (value*1792)/range;

	if (pixel < 256)
		*red = 0, *green = 0, *blue = pixel;
	else if (pixel < 512)
		*red = 0, *green = pixel-256, *blue = 255;
	else if (pixel < 768)
		*red = 0, *green=255, *blue = 255-(pixel-512);
	else if (pixel < 1024)
		*red = pixel-768, *green = 255, *blue = 0;
	else if (pixel < 1280)
		*red = 255, *green = 255-(pixel-1024), *blue = 0;
	else if (pixel < 1536)
		*red = 255, *green = 0, *blue = pixel-1280;
	else *red = 255, *green = pixel-1536, *blue = 255;

	*red *= *red;
	*green *= *green;
	*blue *= *blue;

	if (greyscale)
	{
		int avg = (*red+*green+*blue)/3;
		*red = avg, *green = avg, *blue = avg;
	}
}

void Graph::grayramp(int value, int minalt, int maxalt, ushort *r, ushort *g, ushort *b)
{
#define GAMMA 2.5
	double one_over_gamma, conversion;
	float convconst;
	unsigned short val;

#ifdef _COLOR_CHANNEL_IS_USHORT
	convconst = 65535.0;
#else
	convconst = 255.0;
#endif
	one_over_gamma = 1.0/GAMMA;
	conversion = convconst/pow((double)(maxalt-minalt), one_over_gamma);

	val = (unsigned short)((unsigned)0.5+pow((double)(value-minalt), one_over_gamma)*conversion);

	*r = val, *g = val, *b = val;
#undef GAMMA
}

void Graph::DrawSite(DTEDGrid *g, Site *p, ushort col[3], 
					 float size, bool showlabel)
{
#ifdef _NO_PLOTXY
	return;
#else
	ipoint2_t pt;

	if (p == NULL || m_pHandle.p == NULL || g == NULL)
		return;

	m_pHandle.p->pencolor(col[0], col[1], col[2]);
	m_pHandle.p->fillcolor(col[0], col[1], col[2]);

	g->MapCoordToGraphic(p->Latitude(), p->Longitude(), &pt.x, &pt.y);

	if (pt.x == -1 && pt.y == -1)
	{
		printf("Graph::DrawSite::map error\n");
		p->Print();
	}
	else 
	{
		/*printf("Graph::DrawSite::x = %d, y = %d\n", x, y);*/
		m_pHandle.p->marker(pt.x, pt.y, 2, (int)size);

		if (showlabel)
		{
			/* Show label */
			m_pHandle.p->alabel('c', 'b', (char*)p->Name().c_str());
		}
	}
#endif
}

void Graph::DrawPoint(DTEDGrid *g, Site *p, ushort col[3], float size)
{
#ifdef _NO_PLOTXY
	return;
#else
	ipoint2_t pt;

	if (p == NULL || m_pHandle.p == NULL || g == NULL)
	{
		printf("DrawPoint() error: p = %p, handle = %p, g = %p\n",
			p, m_pHandle.p, g);
		return;
	}

	m_pHandle.p->pencolor(col[0], col[1], col[2]);
	m_pHandle.p->fillcolor(col[0], col[1], col[2]);

	g->MapCoordToGraphic(p->Latitude(), p->Longitude(), &pt.x, &pt.y);

	if (pt.x == -1 && pt.y == -1 && 0)
	{
		printf("Graph::DrawPoint::map error\n");
		p->Print();
		return;
	}
	m_pHandle.p->point(pt.x, pt.y);
#endif
}

/* Takes 2 sites (i.e., points), calculates the slope between the two,
in order to calculate a point perpendicular to the resulting line
segment.  The point is assumed to be perpendicular to the first site,
on the left side. */
void Graph::DrawParallelPoint(DTEDGrid *g, Site *p1, Site *p2, ushort col[3], 
							  float size, int level)
{
#ifdef _NO_PLOTXY
	return;
#else
	ipoint2_t pt1, pt2;
	fpoint2_t npt;

	if (p1 == NULL || p2 == NULL || m_pHandle.p == NULL || g == NULL)
		return;

	m_pHandle.p->pencolor(col[0], col[1], col[2]);
	m_pHandle.p->fillcolor(col[0], col[1], col[2]);

	g->MapCoordToGraphic(p1->Latitude(), p1->Longitude(), &pt1.x, &pt1.y);
	g->MapCoordToGraphic(p2->Latitude(), p2->Longitude(), &pt2.x, &pt2.y);

	if ((pt1.x == -1 && pt1.y == -1) || (pt2.x == -1 && pt2.y == -1))
	{
		printf("Graph::DrawSite::map error\n");
		return;
	}

	{
		float dx, dy;
		float r1, r2;
		double theta1, theta2;

		dx = (float)(pt2.x-pt1.x);
		dy = (float)(pt2.y-pt1.y);

		/* convert p1 -> p2 to polar */
		/* distance from p1 to p2 */
		r1 = sqrt((dx*dx)+(dy*dy));
		/* theta1 is in radians */
		theta1 = atan2(dy, dx);

		/* rotate theta1 90deg right (clockwise) */
		theta2 = theta1+(M_PI/4.0);

		/* distance to next point is level*10 */
		r2 = (level+1)*5.0f;
		/* convert p1 -> p3 to cartesian */
		npt.y = (float)(r2*sin(theta2));
		npt.x = (float)(r2*cos(theta2));
		/* npt.x and npt.y are offsets; apply to p1 */

		/*printf("r1 = %3.2f, theta1 = %3.6f,\tr2 = %3.2f, theta2 = %3.6f,\tnpt.x = %3.2f, npt" \
		".y = %3.2f\n", r1, theta1, r2, theta2, npt.x, npt.y);*/
		npt.x += (float)pt1.x;
		npt.y += (float)pt1.y;

		//cout<<"npt.x: "<<npt.x<<", npt.y: "<<npt.y<<endl;
	}

	m_pHandle.p->fpoint(npt.x, npt.y);
#endif
}

void Graph::DrawParallelLine(DTEDGrid *g, Site *p1, Site *p2, ushort col[3], 
							 float size, int level)
{
#ifdef _NO_PLOTXY
	return;
#else
	ipoint2_t pt1, pt2;
	fpoint2_t npt1, npt2;

	if (p1 == NULL || p2 == NULL || m_pHandle.p == NULL || g == NULL)
		return;

	m_pHandle.p->pencolor(col[0], col[1], col[2]);
	m_pHandle.p->fillcolor(col[0], col[1], col[2]);

	g->MapCoordToGraphic(p1->Latitude(), p1->Longitude(), &pt1.x, &pt1.y);
	g->MapCoordToGraphic(p2->Latitude(), p2->Longitude(), &pt2.x, &pt2.y);

	if ((pt1.x == -1 && pt1.y == -1) || (pt2.x == -1 && pt2.y == -1))
	{
		printf("Graph::DrawSite::map error\n");
		return;
	}

	{
		float dx, dy;
		float r1, r2;
		double theta1, theta2;

		dx = (float)(pt2.x-pt1.x);
		dy = (float)(pt2.y-pt1.y);

		/* convert p1 -> p2 to polar */
		/* distance from p1 to p2 */
		r1 = sqrt((dx*dx)+(dy*dy));
		/* theta1 is in radians */
		theta1 = atan2(dy, dx);

		/* rotate theta1 90deg right (clockwise) */
		theta2 = theta1+(M_PI/4.0f);

		/* distance to next point is level*10 */
		r2 = (level+1)*size;
		/* convert p1 -> p3 to cartesian */
		dy = (float)(r2*sin(theta2));
		dx = (float)(r2*cos(theta2));
		/* dx and dy are offsets; apply to p1 and p2 */

		/*printf("r1 = %3.2f, theta1 = %3.6f,\tr2 = %3.2f, theta2 = %3.6f,\tnx = %3.2f, ny = %3.2f\n",
		r1, theta1, r2, theta2, nx, ny);*/
		npt1.x = dx+(float)pt1.x;
		npt1.y = dy+(float)pt1.y;
		npt2.x = dx+(float)pt2.x;
		npt2.y = dy+(float)pt2.y;

		//cout<<"nx: "<<nx<<", ny: "<<ny<<endl;
	}

	m_pHandle.p->fline(npt1.x, npt1.y, npt2.x, npt2.y);
#endif
}
