#ifndef _graph_h
#define _graph_h

#include <cstdio>
#include <string>
#include "common.h"

#ifdef WIN32
#define _USE_MATH_DEFINES
#include <math.h>
#endif

using namespace std;
#define INCLUDE_PNG_SUPPORT

class DTED;
class DTEDGrid;
class Path;
class Site;
class Collection;
class Plotter;
class XPlotter;
class PNGPlotter;
class GIFPlotter;
class PNMPlotter;
class BMPPlotter;

class Graph
{
public:
	enum { gtX = 0, gtPNG, gtBMP };

	Graph(int type);
	Graph(int type, DTED *d, Collection *sc = NULL);
	Graph(int type, DTEDGrid *g, Collection *sc = NULL);
	~Graph();

	bool Allocate(DTED *d);
	bool Allocate(DTEDGrid *g, int maxwidth, int maxheight);
	bool Generate(DTED *d);
	bool Generate(DTEDGrid *g);
	bool DrawPath(DTED *d, Path *path, ushort col[3]);
	void DrawSite(DTEDGrid *g, Site *p, ushort col[3], 
		float size, bool showlabel);
	void DrawPoint(DTEDGrid *g, Site *p, ushort col[3], 
		float size);
	void DrawParallelPoint(DTEDGrid *g, Site *p1, Site *p2, ushort col[3], 
		float size, int level);
	void DrawParallelLine(DTEDGrid *g, Site *p1, Site *p2, ushort col[3], 
		float size, int level);
	bool Close();

	void LoadContours(string &fname, DTEDGrid *grid, bool tight_bounds);

	void SetCollection(Collection *sc);
	Plotter *Handle();
	bool IsOpen() { return m_bOpen; }

protected:
	bool drawcell(DTED *cell, int xindex, int yindex, int xmin, int ymin, int xmax, int ymax);
	void colorramp(int range, int value, ushort *red, ushort *green, ushort *blue);
	void grayramp(int value, int minalt, int maxalt, ushort *r, ushort *g, ushort *b);
	union
	{
		XPlotter *x;
		PNGPlotter *png;
		BMPPlotter *bmp;
		Plotter *p;
	} m_pHandle;
	Collection *m_pCollection;
	FILE *m_pOutputFile;
	bool m_bOpen;
	int m_iMinAltitude;
	int m_iMaxAltitude;
	int m_iPlotterType;
};

#endif
