#ifndef _greatcircle_h
#define _greatcircle_h

#ifdef WIN32
#define _USE_MATH_DEFINES
#include <math.h>
#endif

/*
Author: James A. Rogers (RogersJA@navair.navy.mil)
Date:   2003-11-19 0946 PST (-8:00)
File:   greatcircle.{cpp,h}

Routines for calculating great circle distances, bearings,
end points, and intermediate points.
*/

#define GC_RADIANS_TO_DEGREES(v) ((180.0/M_PI)*v)
#define GC_DEGREES_TO_RADIANS(v) ((M_PI/180.0)*v)
#define GC_NM_TO_METERS(v)       (v*1852.0)
#define GC_METERS_TO_NM(v)       (v/1852.0)
#define GC_NM_TO_RADIANS(v)      ((M_PI/(10800.0))*v)
#define GC_RADIANS_TO_NM(v)      (((10800.0)/M_PI)*v)

class GreatCircle
{
public:
	static double Distance(double lat1, double lon1,
		double lat2, double lon2,
		bool asRadians);
	static double Radial(double lat1, double lon1,
		double lat2, double lon2,
		bool asRadians);

	static bool IntermediateByDistance(double lat1, double lon1,
		double lat2, double lon2,
		double *latx, double *lonx,
		double interval_dist,
		double total_dist);

	static bool IntermediateByFraction(double lat1, double lon1,
		double lat2, double lon2,
		double *latx, double *lonx,
		double total_dist,
		float fraction);


	static double RadiansToNm(double rad);
	static double NmToRadians(double nm);
	static double NmToMeters(double nm);
	static double MetersToNm(double m);
	static double RadiansToDegrees(double rad);
	static double DegreesToRadians(double deg);
};

#endif
