#ifndef _path_h
#define _path_h

#include <vector>
#include <sstream>

class Coord;
class Site;
class DTED;
class DTEDGrid;

#ifdef WIN32
#define _USE_MATH_DEFINES
#include <math.h>
#endif

using namespace std;

class DTED;
typedef DTED *(*DTEDLookupCallback)(double lat, double lon);

class Path
{
public:
	Path();
	Path(Site *src, Site *dest, DTED *cell, bool interpalt);
	~Path();

	Coord *Location(int index);
	Site *LocationSite(int index);

	int AddPoint(double lat, double lon, double alt, double dis);
	void DelPoint(int index);
	bool CreatePath(Site *src, Site *dest, DTED *cell, DTEDLookupCallback callBack,
		bool include_lastpos, float inc, bool interpalt);
	bool SaveToFile(char *fname);
	bool SaveToFileForGnuplot(char *fname, int pn, int sn);

	double Latitude(int index);
	double Longitude(int index);
	double Altitude(int index);
	double Distance(int index);
	long Obstruction()                                 { return m_lObstruction; }
	int ObstructionAlt()                               { return m_iObstructionAlt; }
	unsigned long Length()                             { return (unsigned long)m_vCoords.size(); }

	void Obstruction(long v)                           { m_lObstruction = v; }
	void ObstructionAlt(int v)                         { m_iObstructionAlt = v; }

	int TravelLOS(Site *src, Site *dest, int &obsalt);

protected:
	vector<Coord*> m_vCoords;
	Site *m_pSource;
	Site *m_pDest;
	long m_lObstruction;
	int m_iObstructionAlt;

private:
	double _getcoordvalue(int index, int field);
	double _arccos(double x, double y);
};

#endif
