#ifndef _radial_h
#define _radial_h

#include <vector>

using namespace std;

class Radial
{
 public:
  struct viewpoint
  {
    struct
    {
      int xi;
      int yi;
      int up;
      int down;
      int left;
      int right;
    } site;
    struct
    {
      int lat;
      int lon;
    } count;
    struct
    {
      int width;
      int height;
    } dim;
    double interval;

    void calc_bound_dist()
    {
      /* determine distances from boundaries to POV */
      site.up = count.lat-site.yi;
      site.down = site.yi;
      site.left = site.xi;
      site.right = count.lon-site.xi;
    }
  };

  typedef struct point
  {
    int x;
    int y;
    struct
    {
      double deg;
      double rad;
    } a;

    void operator=(point &c)
    {
      x = c.x;
      y = c.y;
      a.deg = c.a.deg;
      a.rad = c.a.rad;
    }
    bool operator<(const Radial::point &c) const
    {
      return (a.deg < c.a.deg);
    }
    point()
    {
      x = 0;
      y = 0;
      a.deg = 0.0;
      a.rad = 0.0;
    }
    point(float f)
    {
      x = 0;
      y = 0;
      a.deg = f;
      a.rad = a.deg*(M_PI/180.0);
    }
    void fill(int i1, int i2, float f1)
    {
      x = i1;
      y = i2;
      a.deg = f1;
      a.rad = a.deg*(M_PI/180.0);
    }
  } point;

  
  Radial() : m_vList() { }
  ~Radial() { m_vList.clear(); }

  void Create(int pov_x, int pov_y, float interval, int height, int width,
	      int latcnt, int loncnt);
  
  inline unsigned int Count()                   { return m_vList.size(); }
  Radial::point *Point(int index);
  Radial::point *Point(float bearing);

 protected:
  void _calculate(struct viewpoint *vp, vector<Radial::point> &list);

 private:
  vector<Radial::point> m_vList;

};

#endif
