#ifndef _site_h_
#define _site_h_

/*
Author: James A. Rogers (RogersJA@navair.navy.mil)
Date:   2003-11-19 0926 PST (-8:00)
File:   site.{cpp,h}

%DESCRIPTION%
*/

#include <string>
#include <iostream>
#include <fstream>
#include <sstream>
#include <ctype.h>

using namespace std;

#define TWOPI         6.283185307179586
#define HALFPI        1.570796326794896
#define PI            3.141592653589793
#define DEG2RAD       1.74532925199e-02
#define EARTHRADIUS   20902230.97

class Coord;

#pragma pack(1)
class Site
{
public:
	Site();
	Site(char *name, double lat, double lon, double alt, float ant);
	Site(char *fname);

	double operator-(Site &c);
	double operator-(Coord &c);
	double operator^(Site &c);
	bool operator==(Site &c);

	double Azimuth(Site &dest);
	double Elevation(Site &dest);

	void SetLatitude(int deg, int min, float sec, char dir);
	void SetLongitude(int deg, int min, float sec, char dir);

	void Print(ostream *fp = NULL);

	string &Name() { return m_sName; }
	bool Valid() { return m_bValid; }
	bool Active() { return m_bActive; }
	double Latitude() { return m_dLatitude; }
	double Longitude() { return m_dLongitude; }
	double Altitude() { return m_dAltitude; }
	float Antenna() { return m_fAntenna; }

	void Name(string v) { m_sName = v; }
	void Valid(bool v) { m_bValid = v; }
	void Active(bool v) { m_bActive = v; }
	void Latitude(double v) { m_dLatitude = v; }
	void Longitude(double v) { m_dLongitude = v; }
	void Altitude(double v) { m_dAltitude = v; }
	void Antenna(float v) { m_fAntenna = v; }

protected:
	string m_sName;
	bool m_bValid;
	bool m_bActive;
	double m_dLatitude;
	double m_dLongitude;
	double m_dAltitude;
	float m_fAntenna;
#ifdef _SITE_UTM
	long m_lEasting;
	long m_lNorthing;
	string m_sZone;
	int m_iDatum;
#endif

private:
	bool _valid_lat(double v) { return (-55 < v && v <= 55); }
	bool _valid_lon(double v) { return (-180 <= v && v <= 180); }
	bool _valid_alt(double v) { return (-200 <= v && v < 20000); }
	double _str2lat(char *str, int len);
	double _str2lon(char *str, int len);
};
#pragma pack()

#endif
