#include "utm.h"

/* WGS-84 as default */
int UTM::DatumIndex = 100;

// reference transverse mercator latitude
const double UTM::lat0 = 0.0;
const double UTM::k0 = 0.9996;
const double UTM::Degree = M_PI/180.0;

const struct DATUM UTM::gDatum[] = {
  //     name               ellipsoid   dx      dy       dz
  { "Adindan",                5,   -162,    -12,    206 },	// 0
  { "Afgooye",               15,    -43,   -163,     45 },	// 1
  { "Ain el Abd 1970",       14,   -150,   -251,     -2 },	// 2
  { "Anna 1 Astro 1965",      2,   -491,    -22,    435 },	// 3
  { "Arc 1950",               5,   -143,    -90,   -294 },	// 4
  { "Arc 1960",               5,   -160,     -8,   -300 },	// 5
  { "Ascension Island 58",  14,   -207,    107,     52 },	// 6
  { "Astro B4 Sorol Atoll",  14,    114,   -116,   -333 },	// 7
  { "Astro Beacon E",      14,    145,     75,   -272 },	// 8
  { "Astro DOS 71/4",        14,   -320,    550,   -494 },	// 9
  { "Astronomic Stn 52",    14,    124,   -234,    -25 },	// 10
  { "Australian Geod 66",    2,   -133,    -48,    148 },	// 11
  { "Australian Geod 84",    2,   -134,    -48,    149 },	// 12
  { "Bellevue (IGN)",        14,   -127,   -769,    472 },	// 13
  { "Bermuda 1957",           4,    -73,    213,    296 },	// 14
  { "Bogota Observatory",    14,    307,    304,   -318 },	// 15
  { "Campo Inchauspe",       14,   -148,    136,     90 },	// 16
  { "Canton Astro 1966",     14,    298,   -304,   -375 },	// 17
  { "Cape",                   5,   -136,   -108,   -292 },	// 18
  { "Cape Canaveral",         4,     -2,    150,    181 },	// 19
  { "Carthage",               5,   -263,      6,    431 },	// 20
  { "CH-1903",                3,    674,     15,    405 },	// 21
  { "Chatham 1971",          14,    175,    -38,    113 },	// 22
  { "Chua Astro",            14,   -134,    229,    -29 },	// 23
  { "Corrego Alegre",        14,   -206,    172,     -6 },	// 24
  { "Djakarta (Batavia)",     3,   -377,    681,    -50 },	// 25
  { "DOS 1968",              14,    230,   -199,   -752 },	// 26
  { "Easter Island 1967",    14,    211,    147,    111 },	// 27
  { "European 1950",         14,    -87,    -98,   -121 },	// 28
  { "European 1979",         14,    -86,    -98,   -119 },	// 29
  { "Finland Hayford",       14,    -78,   -231,    -97 },	// 30
  { "Gandajika Base",        14,   -133,   -321,     50 },	// 31
  { "Geodetic Datum 49",    14,     84,    -22,    209 },	// 32
  { "Guam 1963",              4,   -100,   -248,    259 },	// 33
  { "GUX 1 Astro",           14,    252,   -209,   -751 },	// 34
  { "Hjorsey 1955",          14,    -73,     46,    -86 },	// 35
  { "Hong Kong 1963",        14,   -156,   -271,   -189 },	// 36
  { "Indian Bangladesh",      6,    289,    734,    257 },	// 37
  { "Indian Thailand",        6,    214,    836,    303 },	// 38
  { "Ireland 1965",           1,    506,   -122,    611 },	// 39
  { "ISTS 073 Astro 69",    14,    208,   -435,   -229 },	// 40
  { "Johnston Island",       14,    191,    -77,   -204 },	// 41
  { "Kandawala",              6,    -97,    787,     86 },	// 42
  { "Kerguelen Island",      14,    145,   -187,    103 },	// 43
  { "Kertau 1948",            7,    -11,    851,      5 },	// 44
  { "L.C. 5 Astro",           4,     42,    124,    147 },	// 45
  { "Liberia 1964",           5,    -90,     40,     88 },	// 46
  { "Luzon Mindanao",         4,   -133,    -79,    -72 },	// 47
  { "Luzon Philippines",      4,   -133,    -77,    -51 },	// 48
  { "Mahe 1971",              5,     41,   -220,   -134 },	// 49
  { "Marco Astro",           14,   -289,   -124,     60 },	// 50
  { "Massawa",                3,    639,    405,     60 },	// 51
  { "Merchich",               5,     31,    146,     47 },	// 52
  { "Midway Astro 1961",     14,    912,    -58,   1227 },	// 53
  { "Minna",                  5,    -92,    -93,    122 },	// 54
  { "NAD27 Alaska",           4,     -5,    135,    172 },	// 55
  { "NAD27 Bahamas",          4,     -4,    154,    178 },	// 56
  { "NAD27 Canada",           4,    -10,    158,    187 },	// 57
  { "NAD27 Canal Zone",       4,      0,    125,    201 },	// 58
  { "NAD27 Caribbean",        4,     -7,    152,    178 },	// 59
  { "NAD27 Central",          4,      0,    125,    194 },	// 60
  { "NAD27 CONUS",            4,     -8,    160,    176 },	// 61
  { "NAD27 Cuba",             4,     -9,    152,    178 },	// 62
  { "NAD27 Greenland",        4,     11,    114,    195 },	// 63
  { "NAD27 Mexico",           4,    -12,    130,    190 },	// 64
  { "NAD27 San Salvador",     4,      1,    140,    165 },	// 65
  { "NAD83",                 11,      0,      0,      0 },	// 66
  { "Nahrwn Masirah Ilnd",    5,   -247,   -148,    369 },	// 67
  { "Nahrwn Saudi Arbia",     5,   -231,   -196,    482 },	// 68
  { "Nahrwn United Arab",     5,   -249,   -156,    381 },	// 69
  { "Naparima BWI",          14,     -2,    374,    172 },	// 70
  { "Observatorio 1966",     14,   -425,   -169,     81 },	// 71
  { "Old Egyptian",          12,   -130,    110,    -13 },	// 72
  { "Old Hawaiian",           4,     61,   -285,   -181 },	// 73
  { "Oman",                   5,   -346,     -1,    224 },	// 74
  { "Ord Srvy Grt Britn",     0,    375,   -111,    431 },	// 75
  { "Pico De Las Nieves",    14,   -307,    -92,    127 },	// 76
  { "Pitcairn Astro 1967",   14,    185,    165,     42 },	// 77
  { "Prov So Amrican 56",   14,   -288,    175,   -376 },	// 78
  { "Prov So Chilean 63",   14,     16,    196,     93 },	// 79
  { "Puerto Rico",            4,     11,     72,   -101 },	// 80
  { "Qatar National",        14,   -128,   -283,     22 },	// 81
  { "Qornoq",                14,    164,    138,   -189 },	// 82
  { "Reunion",               14,     94,   -948,  -1262 },	// 83
  { "Rome 1940",             14,   -225,    -65,      9 },	// 84
  { "RT 90",                  3,    498,    -36,    568 },	// 85
  { "Santo (DOS)",           14,    170,     42,     84 },	// 86
  { "Sao Braz",              14,   -203,    141,     53 },	// 87
  { "Sapper Hill 1943",      14,   -355,     16,     74 },	// 88
  { "Schwarzeck",            21,    616,     97,   -251 },	// 89
  { "South American 69",    16,    -57,      1,    -41 },	// 90
  { "South Asia",             8,      7,    -10,    -26 },	// 91
  { "Southeast Base",        14,   -499,   -249,    314 },	// 92
  { "Southwest Base",        14,   -104,    167,    -38 },	// 93
  { "Timbalai 1948",          6,   -689,    691,    -46 },	// 94
  { "Tokyo",                  3,   -128,    481,    664 },	// 95
  { "Tristan Astro 1968",    14,   -632,    438,   -609 },	// 96
  { "Viti Levu 1916",         5,     51,    391,    -36 },	// 97
  { "Wake-Eniwetok 60",     13,    101,     52,    -39 },	// 98
  { "WGS 72",                19,      0,      0,      5 },	// 99
  { "WGS 84",                20,      0,      0,      0 },	// 100
  { "Zanderij",              14,   -265,    120,   -358 }		// 101
};

const struct ELLIPSOID UTM::gEllipsoid[] = {
  //      name                               a        1/f
  {  "Airy 1830",                  6377563.396, 299.3249646   },	// 0
  {  "Modified Airy",              6377340.189, 299.3249646   },	// 1
  {  "Australian National",        6378160.0,   298.25        },	// 2
  {  "Bessel 1841",                6377397.155, 299.1528128   },	// 3
  {  "Clarke 1866",                6378206.4,   294.9786982   },	// 4
  {  "Clarke 1880",                6378249.145, 293.465       },	// 5
  {  "Everest (India 1830)",       6377276.345, 300.8017      },	// 6
  {  "Everest (1948)",             6377304.063, 300.8017      },	// 7
  {  "Modified Fischer 1960",      6378155.0,   298.3         },	// 8
  {  "Everest (Pakistan)",         6377309.613, 300.8017      },	// 9
  {  "Indonesian 1974",            6378160.0,   298.247       },	// 10
  {  "GRS 80",                     6378137.0,   298.257222101 },	// 11
  {  "Helmert 1906",               6378200.0,   298.3         },	// 12
  {  "Hough 1960",                 6378270.0,   297.0         },	// 13
  {  "International 1924",         6378388.0,   297.0         },	// 14
  {  "Krassovsky 1940",            6378245.0,   298.3         },	// 15
  {  "South American 1969",        6378160.0,   298.25        },	// 16
  {  "Everest (Malaysia 1969)",    6377295.664, 300.8017      },	// 17
  {  "Everest (Sabah Sarawak)",    6377298.556, 300.8017      },	// 18
  {  "WGS 72",                     6378135.0,   298.26        },	// 19
  {  "WGS 84",                     6378137.0,   298.257223563 },	// 20
  {  "Bessel 1841 (Namibia)",      6377483.865, 299.1528128   },	// 21
  {  "Everest (India 1956)",       6377301.243, 300.8017      }	// 22
};

void UTM::DegToUTM(double lat, double lon, string &zone,
                   double *x, double *y, bool inRadians)
{
  char nz;
  double lon0;
  char zne[8];

  if (inRadians)
    {
      lat*=(180.0/M_PI);
      lon*=(180.0/M_PI);
    }

  if ((lat >= -80.0) && (lat <= 84.0)) {
    nz = 'C'+((short)(lat + 80.0)) / 8;
    if (nz > 'H') ++nz;					// skip 'I' and 'O'
    if (nz > 'N') ++nz;
    if (lat>=0.0)
      nz='N';
    else nz='S';
    lon0 = 6.0 * floor(lon / 6.0) + 3.0;
    sprintf(zne, "%02d%c", ((short)lon0 +183) / 6, nz);
    toTM(lat, lon, lat0, lon0, k0, x, y);
    *x += 500000.0;				// false easting
    if (lat < 0.0)				// false northing for southern hemisphere
      *y = 10000000.0 - *y;
  }
  else {
    strcpy(zne, "00\tx");
    if (lat > 0.0)
      if (lon < 0.0) zne[3] = 'Y';
      else zne[3] = 'Z';
    else
      if (lon < 0.0) zne[3] = 'A';
      else zne[3] = 'B';
    toUPS(lat, lon, x, y);
  }
  zone=zne;
}
//---------------------------------------------------------------------------
void UTM::UTMToDeg(short zone, short southernHemisphere, double x, double y,
		   double *lat, double *lon, bool inRadians)
{
  double lon0;

  if (zone != 0) {
    lon0 = (double)((-183 + 6 * zone));
    if (southernHemisphere)
      y = 1.0e7 - y;			// remove false northing for southern hemisphere
    x -= 500000.0;				//   and false easting
    fromTM(x, y, lat0, lon0, k0, lat, lon);
  }
  else fromUPS(southernHemisphere, x, y, lat, lon);

  if (inRadians)
    {
      *lat*=(M_PI/180.0);
      *lon*=(M_PI/180.0);
    }
}
//---------------------------------------------------------------------------
void UTM::toTM(double lat, double lon, double lat0, double lon0,
               double k0, double *x, double *y)
{
  /*extern struct PREFS gPrefs;*/
  double m, et2, n, t, c, A, a, m0, es, lambda, phi, lambda0, phi0;

  // assume using WGS84 (100)
  datumParams(DatumIndex, &a, &es);

  lambda = lon * Degree;
  phi = lat * Degree;

  phi0 = lat0 * Degree;
  lambda0 = lon0 * Degree;

  m0 = M(phi0, a, es);
  m = M(phi, a, es);

  et2 = es / (1 - es);
  n = a / sqrt(1 - es * pow(sin(phi), 2.0));
  t = pow(tan(phi), 2.0);
  c = et2 * pow(cos(phi), 2.0);
  A = (lambda - lambda0) * cos(phi);
  *x = k0*n*(A + (1.0 - t + c)*A*A*A/6.0
	     + (5.0 - 18.0*t + t*t + 72.0*c - 58.0*et2)*pow(A, 5.0) / 120.0);
  *y = k0*(m - m0 + n*tan(phi)*(A*A/2.0
				+ (5.0 - t + 9.0*c + 4*c*c)*pow(A, 4.0)/24.0
				+ (61.0 - 58.0*t + t*t + 600.0*c - 330.0*et2)*pow(A, 6.0)/720.0) );

}
//---------------------------------------------------------------------------
void UTM::fromTM(double x, double y, double lat0, double lon0, double k0, double *lat, double *lon)
{
  /*extern struct PREFS gFilePrefs;*/
  double a, m0, es, et2, m, e1, mu, phi1, c1, t1, n1, r1, d, phi0, lambda0;

  phi0 = lat0 * Degree;
  lambda0 = lon0 * Degree;

  // assume using WGS84 (100)
  datumParams(DatumIndex, &a, &es);

  m0 = M(phi0, a, es);

  et2 = es / (1.0 - es);
  m = m0 + y / k0;
  e1 = (1.0 - sqrt(1.0 - es)) / (1.0 + sqrt(1.0 - es));
  mu = m / (a * (1.0 - es/4.0 - 3.0 * es*es/64.0 - 5.0 * es*es*es/256.0));
  phi1 = mu + (3.0 * e1/2.0 - 27.0 * pow(e1, 3.0)/32.0) * sin(2.0 * mu)
    + (21.0 * e1*e1/16.0 - 55.0 * pow(e1, 4.0)/32.0)
    * sin(4.0 * mu) + 151.0 * pow(e1, 3.0)/96.0 * sin(6.0 * mu)
    + 1097.0 * pow(e1, 4.0)/512.0 * sin(8.0 * mu);
  c1 = et2 * pow(cos(phi1), 2.0);
  t1 = pow(tan(phi1), 2.0);
  n1 = a / sqrt(1 - es * pow(sin(phi1), 2.0));
  r1 = a * (1.0 - es) / pow(1.0 - es * pow(sin(phi1), 2.0), 1.5);
  d = x / (n1 * k0);
  *lat = (phi1 - n1 * tan(phi1) / r1
	  * (d*d / 2.0 - (5.0 + 3.0 * t1 + 10.0 * c1 - 4.0 * c1*c1 - 9.0 * et2)
	     * pow(d, 4.0) / 24.0 + (61.0 + 90.0 * t1 + 298.0 * c1 + 45.0 * t1*t1
				     - 252.0 * et2 - 3.0 * c1*c1) * pow(d, 6.0) / 720.0 )) / Degree;
  *lon = (lambda0 + (d - (1.0 + 2.0 * t1 + c1) * pow(d, 3.0)/6.0
		     + (5.0 -2.0 * c1 + 28.0 * t1 - 3.0 * c1*c1 + 8.0 * et2 + 24.0 * t1*t1)
		     * pow(d, 5.0)/120.0) / cos(phi1)) / Degree;
}
//---------------------------------------------------------------------------
double UTM::M(double phi, double a, double es)
{
  if (phi == 0.0)
    return 0.0;
  else
    {
      return a * (
		  ( 1.0 - es/4.0 - 3.0*es*es/64.0 - 5.0*es*es*es/256.0 ) * phi -
		  ( 3.0*es/8.0 + 3.0*es*es/32.0 + 45.0*es*es*es/1024.0 ) * sin(2.0 * phi) +
		  ( 15.0*es*es/256.0 + 45.0*es*es*es/1024.0 ) * sin(4.0 * phi) -
		  ( 35.0*es*es*es/3072.0 ) * sin(6.0 * phi) );
    }
}
//---------------------------------------------------------------------------
void UTM::toUPS(double lat, double lon, double *x, double *y)
{
  /*extern struct PREFS gPrefs;*/
  double a, t, e, es, rho;
  const double k0 = 0.994;

  double lambda = lon * Degree;
  double phi = fabs(lat * Degree);

  // assume using WGS84 (100)
  datumParams(DatumIndex, &a, &es);
  e = sqrt(es);
  t = tan(M_PI/4.0 - phi/2.0) / pow( (1.0 - e * sin(phi)) / (1.0 + e * sin(phi)), (e/2.0) );
  rho = 2.0 * a * k0 * t / sqrt(pow(1.0+e, 1.0+e) * pow(1.0-e, 1.0-e));
  *x = rho * sin(lambda);
  *y = rho * cos(lambda);

  if (lat > 0.0)	// Northern hemisphere
    *y = -(*y);
  *x += 2.0e6;	// Add in false easting and northing
  *y += 2.0e6;
}
//---------------------------------------------------------------------------
void UTM::fromUPS(short southernHemisphere, double x, double y,
                  double *lat, double *lon)
{
  /*extern struct PREFS gFilePrefs;*/
  double a, es, e, t, rho;
  const double k0 = 0.994;

  // assume using WGS84 (100)
  datumParams(DatumIndex, &a, &es);
  e = sqrt(es);

  x -= 2.0e6;		// Remove false easting and northing
  y -= 2.0e6;

  rho = sqrt(x*x + y*y);
  t = rho * sqrt(pow(1.0+e, 1.0+e) * pow(1.0-e, 1.0-e)) / (2.0 * a * k0);

  calcPhi(lat, e, t);
  *lat /= Degree;

  if (y != 0.0)
    t = atan(fabs(x/y));
  else {
    t = M_PI / 2.0;
    if (x < 0.0) t = -t;
  }

  if (!southernHemisphere)
    y = -y;

  if (y < 0.0)
    t = M_PI - t;

  if (x < 0.0)
    t = -t;

  *lon = t / Degree;
}
//---------------------------------------------------------------------------
void UTM::calcPhi(double *phi, double e, double t)
{
  double old = M_PI/2.0 - 2.0 * atan(t);
  short maxIterations = 20;

  while ( (fabs((*phi - old) / *phi) > 1.0e-8) && maxIterations-- ) {
    old = *phi;
    *phi = M_PI/ 2.0 - 2.0 * atan( t * pow((1.0 - e * sin(*phi)) / ((1.0 + e * sin(*phi))), (e / 2.0)) );
  }
}
//---------------------------------------------------------------------------
void UTM::datumParams(short datum, double *a, double *es)
{
  // flattening
  double f = 1.0 / gEllipsoid[gDatum[datum].ellipsoid].invf;
  // eccentricity^2
  *es = 2 * f - f*f;
  // semimajor axis
  *a = gEllipsoid[gDatum[datum].ellipsoid].a;
}
//---------------------------------------------------------------------------
