// import_geodetic.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include "geod.h"
#include "collection.h"
#include "site.h"

int main(int argc, char *argv[])
{
	int r;
	char *ifname, *ofname;
	FILE *fpi;
	char buf[1024];
	Collection *col;

	/* check parameters */
	if (argc != 3)
	{
		printf("Usage: %s input-file output-file\n", argv[0]);
		return 1;
	}

	/* get file names */
	ifname = argv[1];
	ofname = argv[2];

	/* open input file */
	fpi = fopen(ifname, "r");

	/* check for errors */
	if (!fpi)
	{
		perror("fopen() input");
		return 1;
	}

	/* allocate collection */
	col = new Collection();

	/* read each line from the input file */
	while (!feof(fpi) && fgets(buf, 1024, fpi) != NULL)
	{
		Site *p;
		char arg0[64], arg1[64], arg2[64];

		/* check for a comment */
		if (buf[0] == '#')
			goto _clear_buf;

		/* remove CR/LF */
		if (buf[strlen(buf)-1] == '\r')
			buf[strlen(buf)-1] = '\0';
		if (buf[strlen(buf)-1] == '\n')
			buf[strlen(buf)-1] = '\0';

		/* try to scan the args */
		r = sscanf(buf, "%[^,],%[^,],%[^,]", arg0, arg1, arg2);

		/* continue if all 3 args not scanned */
		if (r != 3)
			goto _clear_buf;

		/* create new site object */
		p = new Site;

		/* convert latitude */
		p->Latitude(geod::readgeodetic(arg0));

		/* convert longitude */
		p->Longitude(geod::readgeodetic(arg1));

		/* convert altitude from feet */
		p->Altitude(atof(arg2)/3.281);

		/* print site */
		p->Print();

		/* add to collection */
		col->AddSite(p);

_clear_buf:
		/* clear buffer */
		memset(buf, 0, 1024);
	}

	/* close files */
	fclose(fpi);

	/* write collection to output file if at least one site contained */
	if (col->SiteCount() > 0)
		col->SaveSitesToFile(ofname);

	/* free collection */
	delete col;

	/* return success */
	return 0;
}

