// lineofsight.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"

#include <iostream>
#include <string>
#include <sys/stat.h>
#include <sys/types.h>
#include "dtedgrid.h"
#include "dted.h"
#include "site.h"
#include "path.h"
#include "collection.h"
#include "graph_win32.h"
#include "coord.h"

string flight_file;
string emitter_file;
string dted_path;
string pov_file;
string output_textfile;
string contour_file;
bool graph_plot = false;
bool tight_bounds = false;
bool false_color = false;
bool use_contour = false;
bool ignore_cache = true;
bool show_min_coverage = true;
int zoom_level = 1;
int handle = -1;
int max_height = -1;
int max_width = -1;
int line_width = 2;
DTEDGrid *grid;

//#include "dtedcommon.cpp"

void map_color(int i_emt, unsigned short *r, unsigned short *g,
			   unsigned short *b)
{
	switch (i_emt)
	{
	case 0:
		*r = 0xffff, *g = 0xffff, *b = 0xffff;
		break;
	case 1:
		*r = 0, *g = 0, *b = 0xffff;
		break;
	case 2:
		*r = 0xffff, *g = 0xffff, *b = 0;
		break;
	case 3:
		*r = 0, *g = 0xffff, *b = 0xffff;
		break;
	case 4:
		*r = 0xffff, *g = 0, *b = 0xffff;
		break;
	}
}


void calculate_los()
{
	Collection fc;
	Collection ec;
	Graph *g;
	double bxmin, bymin, bxmax, bymax;
	bool b;
	int i, j, fltpathmax, emtpathmax;
	Path ray;
	char buf[64] = "";
	int obsalt, obsidx;

	if (pov_file != "")
	{
		ec.AddSite(new Site((char*)pov_file.c_str()));
		cout<<"Using \""<<ec.Sites(0)->Name()<<"\" as static site\n\n";
	}
	else 
	{
		ec.LoadSitesFromFile((char*)emitter_file.c_str());
		if (pov_file != "")
			ec.AddSite(new Site((char*)pov_file.c_str()));
		cout<<"Emitter file has "<<ec.SiteCount()<<" sites\n";
	}

	fc.LoadSitesFromFile((char*)flight_file.c_str());
	cout<<"Flight file has "<<fc.SiteCount()<<" sites\n";
	fc.CoordBounds(&ec, &bxmin, &bymin, &bxmax, &bymax, false);

	grid = DTEDGrid::Load(bxmin, bymin, bxmax, bymax, dted_path, true);
	if (grid == NULL)
		return;

	fc.Grid(grid);
	ec.Grid(grid);
	b = fc.GeneratePath(0.03f, true);
	if (b == true)
		cout<<"Generated "<<fc.PathCount()<<" paths along flight path\n";
	else printf("Failed to generate paths\n");

	unsigned short color[3] = {0, 0, 0};

	if (graph_plot)
	{
		cout<<"Drawing plot...\n";
		g = new Graph(Graph::gtBMP);
		g->SetCollection(&fc);
		g->Allocate(grid, max_width, max_height);
		if (!g->IsOpen())
		{
			cout<<"Graph not opened!\n";
			return;
		}

		//g->Handle()->pencolor(255, 0, 0);
		//g->Handle()->ellipse(300, 300, 900, 900);
		//g->Close();
		//goto _delete_grid;

		if (false_color)
			g->Generate(grid);

		fc.Handle(g->Handle());
		ec.Handle(g->Handle());
		//fc.GraphSites();

		for (int k = 0; k < (int)ec.SiteCount(); k++)
		{
			map_color(k, &color[0], &color[1], &color[2]);
			g->DrawSite(grid, ec.Sites(k), color, 35.0, true);
		}
	}

	/* run flight paths to backtrace LOS to site pov */
	fltpathmax = fc.PathCount();

	/* run emitter paths to backtrace LOS */
	emtpathmax = ec.SiteCount();
	cout<<"Emitter paths: "<<emtpathmax<<endl;

	/* for each step in the path, check a ray to the static site */
	cout<<"Checking for line-of-sight obstructions...\n";

	if (graph_plot)
		g->Handle()->pencolor(0, 0xffff, 0);

	//g->Close();
	//goto _delete_grid;
	cout<<"flightpathmax: "<<fltpathmax<<endl;

	for (i = 0; i < fltpathmax; i++)
	{
		Path *p = fc.Paths(i);
		Site *s = fc.Sites(i);

		if (i % 100 == 0)
		{
			printf("%d / %d\n", i, fltpathmax);
			fflush(stdout);
		}

		for (j = 0; j < (int)p->Length(); j++)
		{
			Coord *c1 = p->Location(j);

			//printf("path length: %d\n", (int)p->Length());
			for (int i_emt = 0; i_emt < emtpathmax; i_emt++)
			{
				Site *pov = ec.Sites(i_emt);

				if (pov->Active() == false)
				{
					cout<<"Site \""<<pov->Name()<<"\" is inactive\n";
					continue;
				}

				Site dest1((char*)buf, 
					c1->Latitude,
					c1->Longitude,
					s->Altitude(), 0);

				/* create path between the static point 
				and the site destination */
				ray.CreatePath(pov, &dest1, NULL,
					DTEDGrid::_dtedlookupfunc, 
					true, 0.03f, false);

				/* travel the path looking for obstructions */
				obsidx = ray.TravelLOS(pov, &dest1, obsalt);

				if (obsidx == -1)
				{
					color[0] = 0;
					color[1] = 0xffff;
					color[2] = 0;

					if (!show_min_coverage)
						map_color(i_emt, &color[0], &color[1], &color[2]);

					g->DrawPoint(grid, &dest1, color, (float)line_width);

					if (show_min_coverage)
					{
						//printf("i_emt = %d, skipping\n", i_emt);
						break;
					}
				}
				else
				{
					if (i_emt == 0)
					{
						color[0] = 0xffff;
						color[1] = 0;
						color[2] = 0;
						g->DrawPoint(grid, &dest1, color, (float)line_width);
					}
				}
				/*else
				{
				map_color(i_emt, &color[0], 
				&color[1], &color[2]);
				g->DrawParallelPoint(grid, &dest1, &dest2, color, (float)line_width, i_emt);
				//g->DrawPoint(grid, &dest1, color, (float)line_width);
				}*/
			}
		}      
	}

	printf("\n");
	if (graph_plot)
	{
		g->Close();
		delete g;
	}

_delete_grid:
	delete grid;
}


void print_help(char *arg)
{
	printf("Usage: %s -c flight-collection-file [-s pov-site | -e emitter-collection-file] -p dted-path [--graph] [--false-color] [--tight-bounds] [-z zoom-level] [-r contour-file] [--max-width x] [--max-height y] [--line-width n]\n", arg);
}

int parse_args(int argc, char *argv[])
{
	int i = 1;


	while (i < argc)
	{
		if (argv[i][0] == '-')
		{
			switch (argv[i][1])
			{
			case 'c':
				if (i+1 < argc)
				{
					flight_file = argv[i+1];
					i++;
				}
				break;

			case 'e':
				if (i+1 < argc)
				{
					emitter_file = argv[i+1];
					i++;
				}
				break;

			case 'p':
				if (i+1 < argc)
				{
					dted_path = argv[i+1];
					i++;
				}
				break;

			case '-':
				{
					if (!strcmp(argv[i], "--graph"))
						graph_plot = true;
					else if (!strcmp(argv[i], "--tight-bounds"))
						tight_bounds = true;
					else if (!strcmp(argv[i], "--false-color"))
						false_color = true;
					else if (!strcmp("--max-width", argv[i]))
						max_width = atoi(argv[i+1]);
					else if (!strcmp("--max-height", argv[i]))
						max_height = atoi(argv[i+1]);
					else if (!strcmp("--line-width", argv[i]))
						line_width = atoi(argv[i+1]);
					else
					{
						cout<<"Error: unknown flag: "<<argv[i]<<endl;
						return -1;
					}
					break;
				}

			case 'z':
				if (i+1 < argc)
				{
					zoom_level = atoi(argv[i+1]);
					if (zoom_level == 0)
						zoom_level = 1;
					i++;
				}
				break;

			case 'r':
				contour_file = argv[i+1];
				use_contour = true;
				i++;
				break;

			case 's':
				pov_file = argv[i+1];
				i++;
				break;
			}
		}

		i++;
	}

	/* final sanity checks for arguments */
	if (flight_file == "" || dted_path == "")
		return -1;

	if (pov_file == "" && emitter_file == "")
		return -1;

	return 0;
}

int main(int argc, char *argv[])
{
	int r = parse_args(argc, argv);

	if (r == -1)
	{
		print_help(argv[0]);
		return r;
	}

	calculate_los();

	return 0;
}
