// OptLosDisplayDlg.cpp : implementation file
//

#include "stdafx.h"
#include "optlos.h"
#include "OptLosDisplayDlg.h"
#include ".\optlosdisplaydlg.h"

#include "plotter.h"

// COptLosDisplayDlg dialog

IMPLEMENT_DYNAMIC(COptLosDisplayDlg, CDialog)
COptLosDisplayDlg::COptLosDisplayDlg(CWnd* pParent)
	: CDialog(COptLosDisplayDlg::IDD, pParent),
	m_pPlotter(NULL),
	m_fMapScale(1.0),
	m_tMapOffset(0, 0),
	m_bMouseDown(false),
	m_tMouseDownPos(0, 0)
{
}

COptLosDisplayDlg::~COptLosDisplayDlg()
{
	/* check if bitmap needs to be freed */
	if (m_Bitmap.m_hObject)
		m_Bitmap.DeleteObject();

	/* check if DC needs to be freed */
	if (m_hImageDC.m_hDC)
		m_hImageDC.DeleteDC();
}

void COptLosDisplayDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
}


BEGIN_MESSAGE_MAP(COptLosDisplayDlg, CDialog)
	ON_WM_PAINT()
	ON_WM_KEYUP()
	ON_WM_TIMER()
END_MESSAGE_MAP()


// COptLosDisplayDlg message handlers

BOOL COptLosDisplayDlg::PreTranslateMessage(MSG* pMsg)
{
	/* ignore RETURN and ESCAPE (don't close dialog like default) */
	if (pMsg->message == WM_KEYDOWN)
	{
		switch ((int)pMsg->wParam)
		{
		case VK_RETURN:
		case VK_ESCAPE:
			return true;
		}
	}

	return CDialog::PreTranslateMessage(pMsg);
}

void COptLosDisplayDlg::OnPaint()
{
	if (!m_hImageDC.m_hDC)
		return;

	CPaintDC dc(this); // device context for painting

	/* copy image */
	POINT dim;
	POINT off;

	/* calculate new image dimensions */
	dim.x = (int)((float)m_pPlotter->Width()*m_fMapScale);
	dim.y = (int)((float)m_pPlotter->Height()*m_fMapScale);

	/* calculate image offset */
	off.x = m_tMapOffset.x;
	off.y = m_tMapOffset.y;

	/* set mode */
	dc.SetStretchBltMode(COLORONCOLOR);

	/* draw image */
	dc.StretchBlt(off.x, off.y, dim.x, dim.y, &m_hImageDC, 0, 0, 
				  m_pPlotter->Width(), m_pPlotter->Height(), SRCCOPY);

	// Do not call CWnd::OnPaint() for painting messages
}

void COptLosDisplayDlg::OnKeyUp(UINT nChar, UINT nRepCnt, UINT nFlags)
{
	bool changed;
	CString str;

	/* nothing to do yet */
	changed = false;

	/* check if this is a zoom */
	switch (nChar)
	{
	case VK_ADD:	/* zoom in */
		/* change map scale */
		m_fMapScale += (float)0.1;

		/* stop at 0.5x */
		if (m_fMapScale < (float)0.5)
			m_fMapScale = (float)0.5;

		/* changed a value */
		changed = true;

		break;

	case VK_SUBTRACT:	/* zoom out */
		/* change map scale */
		m_fMapScale -= (float)0.1;

		/* stop at 2x */
		if (m_fMapScale > (float)2.0)
			m_fMapScale = (float)2.0;

		/* changed a value */
		changed = true;

		break;

	case VK_LEFT:		/* pan left */
		/* pan image left */
		m_tMapOffset.x += 15;

		/* changed a value */
		changed = true;

		break;

	case VK_RIGHT:		/* pan right */
		/* pan image right */
		m_tMapOffset.x -= 15;

		/* changed a value */
		changed = true;

		break;
		
	case VK_UP:			/* pan up */
		/* pan image up */
		m_tMapOffset.y += 15;

		/* changed a value */
		changed = true;

		break;

	case VK_DOWN:		/* pan down */
		/* pan image down */
		m_tMapOffset.y -= 15;

		/* changed a value */
		changed = true;

		break;
	}

	/* check if something to do */
	if (changed)
	{
		/* show zoom level */
		str.Format("%3.1f%%", m_fMapScale*100.0);
		SetWindowText(str);

		/* redraw display */
		this->RedrawWindow(NULL, NULL, RDW_ERASE | RDW_INVALIDATE);
	}
}

BOOL COptLosDisplayDlg::OnInitDialog(void)
{
	CString str;

	CDialog::OnInitDialog();

	/* show zoom level */
	str.Format("%3.1f%%", m_fMapScale*100.0);
	SetWindowText(str);

	return 0;
}

void COptLosDisplayDlg::OnTimer(UINT_PTR nIDEvent)
{
	if (nIDEvent == 1)
	{
		/* copy DC from plotter */
		m_hImageDC.BitBlt(0, 0, m_pPlotter->Width(), m_pPlotter->Height(),
						  m_pPlotter->GetDC(), 0, 0, SRCCOPY);

		/* repaint window */
		this->RedrawWindow(NULL, NULL, RDW_INVALIDATE);
	}
}

void COptLosDisplayDlg::SetPlotter(BMPPlotter *p)
{
	/* set local copy of the plotter */
	m_pPlotter = p;

	/* allocate new DC */
	m_hImageDC.CreateCompatibleDC(NULL);

	/* create a new bitmap */
	m_Bitmap.CreateBitmap(p->Width(), p->Height(), 1, 32, NULL);

	/* select new bitmap */
    m_hImageDC.SelectObject(&m_Bitmap);

	/* set timer for copying image */
	SetTimer(1, 1000, NULL);
}
