#include "graphbase_win32.h"
#include "dted.h"

#include "plotter.h"

//#define _COLOR_CHANNEL_IS_USHORT

GraphBase::GraphBase(int type)
{
	m_iPlotterType = type;
	m_pHandle.p = NULL;
	m_bOpen = false;
	m_bGrayScale = true;
	m_iMinAltitude = 0;
	m_iMaxAltitude = 0;
	m_dGamma = 2.5;
}

GraphBase::~GraphBase()
{
	if (m_pHandle.p != NULL && m_bOpen == true)
	{
		m_pHandle.p->closepl();
		delete m_pHandle.p;
		m_pHandle.p = NULL;
	}
}

Plotter *GraphBase::Handle()
{
	return m_pHandle.p;
}

bool GraphBase::Close()
{
	if (m_pHandle.p == NULL)
		return false;

	/* close plotter */
	m_pHandle.p->closepl();

	/* free and clear plotter */
	delete m_pHandle.p;
	m_pHandle.p = NULL;

	if (m_iPlotterType == gtPNG || m_iPlotterType == gtBMP)
		fclose(m_pOutputFile);

	m_bOpen = false;

	return true;
}

bool GraphBase::DrawCell(DTED *cell, int xindex, int yindex)
{
	int x_last, y_last, x, y, px, py;
	int diffalt;
	unsigned short color[3];
	long xoff, yoff;
	short pc;

	if (m_bOpen == false || m_pHandle.p == NULL)
		return false;

	x_last = cell->LongitudeCount();
	y_last = cell->LatitudeCount();

	xoff = xindex*x_last;
	yoff = yindex*y_last;

	if (cell->PlotFake() == true)
		return true;

	if (x_last != y_last || x_last == 0)
	{
		cout<<"GraphBase::drawcell() error: dimensions unequal or zero\n";
		return false;
	}

	m_pHandle.p->pencolor(0xffff, 0, 0);

	if (!m_bGrayScale)
		diffalt = m_iMaxAltitude-m_iMinAltitude;

	for (x = 0; x < x_last; x++)
	{
		for (y = 0; y < y_last; y++)
		{
			px = xoff+x;
			py = yoff+y;

			pc = cell->AltitudeByPost(x, y);
			if (m_bGrayScale)
				GrayRamp(pc, m_iMinAltitude, m_iMaxAltitude, &color[0], &color[1], &color[2]);
			else ColorRamp(diffalt, pc-m_iMinAltitude, &color[0], &color[1], &color[2]);

			/* handle the oceans */
			if (pc == 0)
			{
				color[0] = 0;
				color[1] = 0;
				color[2] = 255;
			}

			m_pHandle.p->pencolor(color[0], color[1], color[2]);
			m_pHandle.p->point(px, py);
		}
	}

	return true;
}

bool GraphBase::DrawCellOffset(DTED *cell, int xorg, int yorg)
{
	int x_last, y_last, x, y, px, py;
	int diffalt;
	unsigned short color[3];
	short pc;

	if (m_bOpen == false || m_pHandle.p == NULL)
		return false;

	m_pHandle.p->pencolor(0xffff, 0, 0);

	if (!m_bGrayScale)
		diffalt = m_iMaxAltitude-m_iMinAltitude;

	x_last = cell->LongitudeCount();
	y_last = cell->LatitudeCount();

	for (x = 0; x < x_last; x++)
	{
		for (y = 0; y < y_last; y++)
		{
			px = xorg+x;
			py = yorg+y;

			pc = cell->AltitudeByPost(x, y);
			if (m_bGrayScale)
				GrayRamp(pc, m_iMinAltitude, m_iMaxAltitude, &color[0], &color[1], &color[2]);
			else ColorRamp(diffalt, pc-m_iMinAltitude, &color[0], &color[1], &color[2]);

			/* handle the oceans */
			if (pc == 0)
			{
				color[0] = 0;
				color[1] = 0;
				color[2] = 255;
			}

			m_pHandle.p->pencolor(color[0], color[1], color[2]);
			m_pHandle.p->point(px, py);
		}
	}

	return true;
}

void GraphBase::ColorRamp(int range, int value, unsigned short *red, unsigned short *green, unsigned short *blue)
{
	bool greyscale = false;
	int pixel;

	if (range == 0)
		range = 1;

	/* this will need to be fixed for Death Valley */
#if 1
	if (value < 0)
		value = 0;
#endif

	/* this shouldn't happen... */
	if (value > range)
		value = range;

	pixel = (value*1792)/range;

	if (pixel < 256)
		*red = 0, *green = 0, *blue = pixel;
	else if (pixel < 512)
		*red = 0, *green = pixel-256, *blue = 255;
	else if (pixel < 768)
		*red = 0, *green=255, *blue = 255-(pixel-512);
	else if (pixel < 1024)
		*red = pixel-768, *green = 255, *blue = 0;
	else if (pixel < 1280)
		*red = 255, *green = 255-(pixel-1024), *blue = 0;
	else if (pixel < 1536)
		*red = 255, *green = 0, *blue = pixel-1280;
	else *red = 255, *green = pixel-1536, *blue = 255;

	*red *= *red;
	*green *= *green;
	*blue *= *blue;

	if (greyscale)
	{
		int avg = (*red+*green+*blue)/3;
		*red = avg, *green = avg, *blue = avg;
	}
}

void GraphBase::GrayRamp(int value, int minalt, int maxalt, unsigned short *r, unsigned short *g, unsigned short *b)
{
	double one_over_gamma, conversion;
	float convconst;
	unsigned short val;

#ifdef _COLOR_CHANNEL_IS_USHORT
	convconst = 65535.0;
#else
	convconst = 255.0;
#endif
	one_over_gamma = 1.0/m_dGamma;
	conversion = convconst/pow((double)(maxalt-minalt), one_over_gamma);

	val = (unsigned short)((unsigned)0.5+pow((double)(value-minalt), one_over_gamma)*conversion);

	*r = val, *g = val, *b = val;
}
