#include "graphdtedgrid_win32.h"
#include "dted.h"
#include "dtedgrid.h"
#include "path.h"
#include "coord.h"
#include "site.h"

#include "plotter.h"

//#define _COLOR_CHANNEL_IS_USHORT

GraphDtedGrid::GraphDtedGrid(int type, DTEDGrid *g) : GraphBase(type),
	m_pGrid(g)
{
	Allocate(-1, -1);
}

GraphDtedGrid::~GraphDtedGrid()
{
	/* do not free the DTED pointer, as we don't own it */
}

bool GraphDtedGrid::Allocate(int maxwidth, int maxheight)
{
#ifdef _NO_PLOTXY
	return true;
#else
	int xsz = WINDOW_SIZE, ysz = WINDOW_SIZE;
	char buf[64];
	float xn = 1.0, yn = 1.0;
	int x_max, y_max;
	DTEDGrid::StringCallback p;
	void *d_ptr;
	char str[512];

	if (m_pGrid == NULL)
		return false;

	x_max = m_pGrid->BoundWidth();
	y_max = m_pGrid->BoundHeight();

	p = m_pGrid->GetStatusCallback(&d_ptr);

	/* calculates necessary window size, including tight
	bounds and compensating for number size of grid */
	m_pGrid->CalcWindowSize(xsz, ysz, xn, yn);

	printf("\txn = %f, yn = %f\n", xn, yn);
	if (xn == 1.0 && yn == 1.0)
		printf("using unscaled (%d, %d, %d, %d)\n", 0, 0, x_max, y_max);
	else
	{
		printf("using scaled (%d, %d, %f, %f)\n", 0, 0, x_max*xn, y_max*yn);
		x_max = (int)((float)x_max*xn);
		y_max = (int)((float)y_max*yn);
	}

	//xsz *= 2;
	//ysz *= 2;

	if (maxwidth != -1 && maxwidth < xsz)
	{
		float scale = 1.0f-((float)xsz/(float)maxwidth);
		printf("greater than maxwidth of %d; scaling by %f\n", maxwidth, scale);
		xsz = maxwidth;
		ysz -= (int)((float)ysz*fabs(scale));
	}
	if (maxheight != -1 && maxheight < ysz)
	{
		float scale = 1.0f-((float)ysz/(float)maxheight);
		printf("greater than maxheight of %d; scaling by %f\n", maxheight, scale);
		xsz -= (int)((float)xsz*fabs(scale));
		ysz = maxheight;
	}

	//xsz = maxwidth;
	//ysz = maxheight;
	_snprintf(buf, 64-1, "%dx%d", xsz, ysz);
	Plotter::parampl("BITMAPSIZE", buf);

	if (p && d_ptr)
	{
		_snprintf(str, 512, "using %dx%d for window size", xsz, ysz);
		p(d_ptr, str);
	}

	switch (m_iPlotterType)
	{
	case gtX: 
		m_pHandle.x = new XPlotter(stdin);
		break;

	case gtPNG:
		m_pOutputFile = fopen("plot.png", "wb+");
		m_pHandle.png = new PNGPlotter(m_pOutputFile);
		break;

	case gtBMP:
		m_pOutputFile = fopen("plot.bmp", "wb+");
		m_pHandle.bmp = new BMPPlotter(m_pOutputFile);
		break;

	default:
		cout<<"GraphDtedGrid::Allocate(): Invalid plotter type specified: "<<m_iPlotterType<<endl;
		return false;
	}

	if (m_pHandle.p->openpl() < 0)
	{
		m_bOpen = false;
		return false;
	}
	m_bOpen = true;

	m_pHandle.p->bgcolor(0xaaaa, 0xaaaa, 0xaaaa);

	ipoint2_t p1;
	ipoint2_t p2;

	p1.x = -1;
	p1.y = -1;
	p2.x = -1;
	p2.y = -1;

	//x_max = maxwidth;
	//y_max = maxheight;

	if (p1.x == -1)
		p1.x = 0;
	if (p1.y == -1)
		p1.y = 0;
	if (p2.x == -1)
		p2.x = x_max;
	if (p2.y == -1)
		p2.y = y_max;
	m_pHandle.p->space (p1.x, p1.y, p2.x, p2.y);

	if (p && d_ptr)
	{
		_snprintf(str, 512, "using mapping for grid: (%d, %d, %d, %d)", 
				p1.x, p1.y, p2.x, p2.y);
		p(d_ptr, str);
	}

	m_pHandle.p->erase();

	return true;
#endif
}

bool GraphDtedGrid::Generate()
{
#ifdef _NO_PLOTXY
	return true;
#else
	int x, y;
	ipoint2_t p1, p2;
	DTEDGrid::StringCallback scb;
	void *d_ptr;
	char buf[512];

	if (m_pGrid == NULL)
		return false;

	scb = m_pGrid->GetStatusCallback(&d_ptr);

	p1.x = -1;
	p1.y = -1;
	p2.x = -1;
	p2.y = -1;

	m_iMinAltitude = m_pGrid->MinAltitude();
	m_iMaxAltitude = m_pGrid->MaxAltitude();

	if (scb && d_ptr)
	{
		_snprintf(buf, 512, "GraphDtedGrid bounds: (%d, %d), (%d, %d)", p1.x, p1.y, p2.x, p2.y);
		scb(d_ptr, buf);

		_snprintf(buf, 512, "GraphDtedGrid max altitude: %d", m_iMaxAltitude);
		scb(d_ptr, buf);

		_snprintf(buf, 512, "GraphDtedGrid min altitude: %d", m_iMinAltitude);
		scb(d_ptr, buf);
	}

	for (x = 0; x < m_pGrid->Width(); x++)
	{
		for (y = 0; y < m_pGrid->Height(); y++)
		{
			DTED *p;
			POINT pt;

			if (scb && d_ptr)
			{
				_snprintf(buf, 512, "drawing cell %d/%d", (x*m_pGrid->Height())+y+1, 
							m_pGrid->Width()*m_pGrid->Height());
				scb(d_ptr, buf);
			}

			p = m_pGrid->Cell(x, y);

			/* top left coordinate of cell */
	        pt.x = p->LonOffset()*(p->LongitudeCount()-1);
			pt.y = p->LatOffset()*(p->LatitudeCount()-1);

			//DrawCell(p, x, m_pGrid->Height()-y-1, -1, -1, -1, -1);
			DrawCellOffset(p, pt.x, pt.y);
			//break;
		}
	}

	return true;
#endif
}

bool GraphDtedGrid::DrawPath(Path *path, unsigned short col[3], int size)
{
	if (m_pGrid == NULL || path == NULL || m_pHandle.p == NULL || m_bOpen == false)
		return false;

	int i;
	Coord *p1;
	Coord *p2;

	m_pHandle.p->pencolor(col[0], col[1], col[2]);
	m_pHandle.p->linewidth(size);

	for (i = 0; i < (int)path->Length()-1; i++)
	{
		ipoint2_t ip1 = {0, 0}, ip2 = {0, 0};

		p1 = path->Location(i);
		p2 = path->Location(i+1);

		m_pGrid->MapCoordToGraphic(p1->Latitude, p1->Longitude, &ip1.x, &ip1.y);
		m_pGrid->MapCoordToGraphic(p2->Latitude, p2->Longitude, &ip2.x, &ip2.y);

		m_pHandle.p->line(ip1.x, ip1.y, ip2.x, ip2.y);
	}

	return true;
}

void GraphDtedGrid::DrawSite(Site *p, unsigned short col[3], float size, bool showlabel)
{
	ipoint2_t pt;

	if (m_pGrid == NULL || p == NULL || m_pHandle.p == NULL)
		return;

	m_pHandle.p->pencolor(col[0], col[1], col[2]);
	m_pHandle.p->fillcolor(col[0], col[1], col[2]);

	m_pGrid->MapCoordToGraphic(p->Latitude(), p->Longitude(), &pt.x, &pt.y);

	if (pt.x == -1 && pt.y == -1)
	{
		printf("GraphDtedGrid::DrawSite::map error\n");
		p->Print();
	}
	else 
	{
		/*printf("GraphDtedGrid::DrawSite::x = %d, y = %d\n", x, y);*/
		m_pHandle.p->marker(pt.x, pt.y, 2, (int)size);

		if (showlabel)
		{
			/* Show label */
			m_pHandle.p->alabel('c', 'b', (char*)p->Name().c_str());
		}
	}
}

void GraphDtedGrid::DrawPoint(Site *p, unsigned short col[3], float size)
{
	ipoint2_t pt;

	if (m_pGrid == NULL || p == NULL || m_pHandle.p == NULL)
	{
		printf("DrawPoint() error: p = %p, handle = %p, m_pGrid = %p\n",
			p, m_pHandle.p, m_pGrid);
		return;
	}

	m_pHandle.p->pencolor(col[0], col[1], col[2]);
	m_pHandle.p->fillcolor(col[0], col[1], col[2]);

	m_pGrid->MapCoordToGraphic(p->Latitude(), p->Longitude(), &pt.x, &pt.y);

	if (pt.x == -1 && pt.y == -1 && 0)
	{
		printf("GraphDtedGrid::DrawPoint::map error\n");
		p->Print();
		return;
	}

#ifdef _USE_POINT
	m_pHandle.p->point(pt.x, pt.y);
#else
	int sz, sz_2;
	RECT r;

	sz = (int)size;
	sz_2 = sz/2;
	r.left = pt.x-sz_2;
	r.right = pt.x+sz_2;
	r.bottom = pt.y+sz_2;
	r.top = pt.y-sz_2;

	m_pHandle.p->ellipse(r.left, r.top, r.right, r.bottom);
#endif
}

/* Takes 2 sites (i.e., points), calculates the slope between the two,
in order to calculate a point perpendicular to the resulting line
segment.  The point is assumed to be perpendicular to the first site,
on the left side. */
void GraphDtedGrid::DrawParallelPoint(Site *p1, Site *p2, unsigned short col[3], 
										float size, int level)
{
	ipoint2_t pt1, pt2;
	fpoint2_t npt;

	if (m_pGrid == NULL || p1 == NULL || p2 == NULL || m_pHandle.p == NULL)
		return;

	m_pHandle.p->pencolor(col[0], col[1], col[2]);
	m_pHandle.p->fillcolor(col[0], col[1], col[2]);

	m_pGrid->MapCoordToGraphic(p1->Latitude(), p1->Longitude(), &pt1.x, &pt1.y);
	m_pGrid->MapCoordToGraphic(p2->Latitude(), p2->Longitude(), &pt2.x, &pt2.y);

	if ((pt1.x == -1 && pt1.y == -1) || (pt2.x == -1 && pt2.y == -1))
	{
		printf("GraphDtedGrid::DrawSite::map error\n");
		return;
	}

	{
		float dx, dy;
		float r1, r2;
		double theta1, theta2;

		dx = (float)(pt2.x-pt1.x);
		dy = (float)(pt2.y-pt1.y);

		/* convert p1 -> p2 to polar */
		/* distance from p1 to p2 */
		r1 = sqrt((dx*dx)+(dy*dy));
		/* theta1 is in radians */
		theta1 = atan2(dy, dx);

		/* rotate theta1 90deg right (clockwise) */
		theta2 = theta1+(M_PI/4.0);

		/* distance to next point is level*10 */
		r2 = (level+1)*5.0f;
		/* convert p1 -> p3 to cartesian */
		npt.y = (float)(r2*sin(theta2));
		npt.x = (float)(r2*cos(theta2));
		/* npt.x and npt.y are offsets; apply to p1 */

		/*printf("r1 = %3.2f, theta1 = %3.6f,\tr2 = %3.2f, theta2 = %3.6f,\tnpt.x = %3.2f, npt" \
		".y = %3.2f\n", r1, theta1, r2, theta2, npt.x, npt.y);*/
		npt.x += (float)pt1.x;
		npt.y += (float)pt1.y;

		//cout<<"npt.x: "<<npt.x<<", npt.y: "<<npt.y<<endl;
	}

	m_pHandle.p->fpoint(npt.x, npt.y);
}

void GraphDtedGrid::DrawParallelLine(Site *p1, Site *p2, unsigned short col[3], 
							 float size, int level)
{
	ipoint2_t pt1, pt2;
	fpoint2_t npt1, npt2;

	if (m_pGrid == NULL || p1 == NULL || p2 == NULL || m_pHandle.p == NULL)
		return;

	m_pHandle.p->pencolor(col[0], col[1], col[2]);
	m_pHandle.p->fillcolor(col[0], col[1], col[2]);

	m_pGrid->MapCoordToGraphic(p1->Latitude(), p1->Longitude(), &pt1.x, &pt1.y);
	m_pGrid->MapCoordToGraphic(p2->Latitude(), p2->Longitude(), &pt2.x, &pt2.y);

	if ((pt1.x == -1 && pt1.y == -1) || (pt2.x == -1 && pt2.y == -1))
	{
		printf("GraphDtedGrid::DrawSite::map error\n");
		return;
	}

	{
		float dx, dy;
		float r1, r2;
		double theta1, theta2;

		dx = (float)(pt2.x-pt1.x);
		dy = (float)(pt2.y-pt1.y);

		/* convert p1 -> p2 to polar */
		/* distance from p1 to p2 */
		r1 = sqrt((dx*dx)+(dy*dy));
		/* theta1 is in radians */
		theta1 = atan2(dy, dx);

		/* rotate theta1 90deg right (clockwise) */
		theta2 = theta1+(M_PI/4.0f);

		/* distance to next point is level*10 */
		r2 = (level+1)*size;
		/* convert p1 -> p3 to cartesian */
		dy = (float)(r2*sin(theta2));
		dx = (float)(r2*cos(theta2));
		/* dx and dy are offsets; apply to p1 and p2 */

		/*printf("r1 = %3.2f, theta1 = %3.6f,\tr2 = %3.2f, theta2 = %3.6f,\tnx = %3.2f, ny = %3.2f\n",
		r1, theta1, r2, theta2, nx, ny);*/
		npt1.x = dx+(float)pt1.x;
		npt1.y = dy+(float)pt1.y;
		npt2.x = dx+(float)pt2.x;
		npt2.y = dy+(float)pt2.y;

		//cout<<"nx: "<<nx<<", ny: "<<ny<<endl;
	}

	m_pHandle.p->fline(npt1.x, npt1.y, npt2.x, npt2.y);
}

bool GraphDtedGrid::Close()
{
	if (m_pGrid != NULL)
	{
		DTEDGrid::StringCallback p;
		void *d_ptr;

		p = m_pGrid->GetStatusCallback(&d_ptr);

		if (p && d_ptr)
		{
			p(d_ptr, "Closing graph and writing to disk");
		}
	}

	return GraphBase::Close();
}