#include <stdlib.h>
#include <stdio.h>
#include "plotter.h"

std::map<std::string,std::string> Plotter::m_mParams;
int Plotter::m_iWidth = -1;
int Plotter::m_iHeight = -1;

Plotter::Plotter(FILE *stream)
{
	m_pFile = stream;
	m_pHandle = NULL;
	m_iLineWidth = 2;
}

Plotter::~Plotter()
{
	/* check if bitmap needs to be freed */
	if (m_hBitmap.m_hObject)
		m_hBitmap.DeleteObject();

	/* check if DC needs to be freed */
	if (m_hImageDC.m_hDC)
		m_hImageDC.DeleteDC();
}

int Plotter::openpl()
{
	std::string str;

	if (m_iWidth == -1 && m_iHeight == -1)
	{
		int w, h, r;

		str = m_mParams["BITMAPSIZE"];
		r = sscanf(str.c_str(), "%dx%d", &w, &h);

		if (r != 2)
			return -1;

		m_iWidth = w;
		m_iHeight = h;
	}

	/* allocate new DC */
	m_hImageDC.CreateCompatibleDC(NULL);

	/* create a new bitmap */
	m_hBitmap.CreateBitmap(m_iWidth, m_iHeight, 1, 32, NULL);

	/* select new bitmap */
    m_hImageDC.SelectObject(&m_hBitmap);

	return 0;
}

void Plotter::closepl()
{
	if (m_hBitmap.m_hObject && m_pFile)
		WriteStream();

	if (m_hBitmap.m_hObject)
		m_hBitmap.DeleteObject();

	if (m_hImageDC.m_hDC)
		m_hImageDC.DeleteDC();
}

void Plotter::pencolor(int r, int g, int b)
{
	m_rPenColor = RGB(r, g, b);
}

void Plotter::fillcolor(int r, int g, int b)
{
	m_rFillColor = RGB(r, g, b);
}

void Plotter::bgcolor(int r, int g, int b)
{
	m_rBackgroundColor = RGB(r, g, b);
}

void Plotter::point(int x, int y)
{
	COLORREF c;

	if (!m_hImageDC.m_hDC)
		return;

	y = m_iHeight-1-y;
	c = m_hImageDC.SetPixel(x, y, m_rPenColor);
}

void Plotter::fpoint(float fx, float fy)
{
	int y;

	if (!m_hImageDC.m_hDC)
		return;

	y = m_iHeight-1-(int)fy;
	m_hImageDC.SetPixelV((int)fx, y, m_rPenColor);
}

void Plotter::marker(int x, int y, int type, int sz)
{
	int half;

	if (!m_hImageDC.m_hDC)
		return;

	m_hImageDC.SelectObject(GetStockObject(DC_PEN));

	/* set pen color */
	m_hImageDC.SetDCPenColor(m_rPenColor);

	half = sz/2;
	y = m_iHeight-1-y;
	m_hImageDC.MoveTo(x - half, y); 
	m_hImageDC.LineTo(x + half, y); 
	m_hImageDC.MoveTo(x, y - half); 
	m_hImageDC.LineTo(x, y + half); 
	m_hImageDC.MoveTo(x, y);
}

void Plotter::space(int xmin, int ymin, int xmax, int ymax)
{
	if (!m_hImageDC.m_hDC)
		return;

	printf("space(): using 0->%d for x, 0->%d for y\n",
		xmax-xmin, ymax-ymin);
}

void Plotter::fspace(float fxmin, float fymin, float fxmax, float fymax)
{
	if (!m_hImageDC.m_hDC)
		return;
}

void Plotter::erase()
{
	std::string str;

	if (!m_hImageDC.m_hDC)
		return;

	if (m_iWidth == -1 && m_iHeight == -1)
	{
		int w, h, r;

		str = m_mParams["BITMAPSIZE"];
		r = sscanf(str.c_str(), "%dx%d", &w, &h);

		if (r != 2)
			return;

		m_iWidth = w;
		m_iHeight = h;
	}

	/* set brush color */
	m_hImageDC.SelectObject(GetStockObject(DC_BRUSH));
	m_hImageDC.SetDCBrushColor(m_rBackgroundColor);

	/* set pen color */
	m_hImageDC.SelectObject(GetStockObject(DC_PEN));
	m_hImageDC.SetDCPenColor( m_rBackgroundColor);

	/* draw point */
	m_hImageDC.Rectangle(0, 0, m_iWidth, m_iHeight);
}

void Plotter::line(int x1, int y1, int x2, int y2)
{
	if (!m_hImageDC.m_hDC)
		return;

	CPen pen;

	/* create pen */
	pen.CreatePen(PS_SOLID, m_iLineWidth, m_rPenColor);

	/* select pen for use */
	m_hImageDC.SelectObject(&pen);

	/* draw point */
	y1 = m_iHeight-1-y1;
	y2 = m_iHeight-1-y2;
	m_hImageDC.MoveTo(x1, y1);
	m_hImageDC.LineTo(x2, y2);
}

void Plotter::fline(float fx1, float fy1, float fx2, float fy2)
{
	int y1, y2;

	if (!m_hImageDC.m_hDC)
		return;

	CPen pen;

	/* create pen */
	pen.CreatePen(PS_SOLID, m_iLineWidth, m_rPenColor);

	/* select pen for use */
	m_hImageDC.SelectObject(&pen);

	/* draw point */
	y1 = m_iHeight-1-(int)fy1;
	y2 = m_iHeight-1-(int)fy2;
	m_hImageDC.MoveTo((int)fx1, y1);
	m_hImageDC.LineTo((int)fx2, y2);
}

void Plotter::linewidth(int sz)
{
	/* set line width */
	m_iLineWidth = sz;
}

void Plotter::rectangle(int x1, int y1, int x2, int y2)
{
	if (!m_hImageDC.m_hDC)
		return;

	/* set brush color */
	m_hImageDC.SelectObject(GetStockObject(DC_BRUSH));
	m_hImageDC.SetDCBrushColor(m_rFillColor);

	/* set pen color */
	m_hImageDC.SelectObject(GetStockObject(DC_PEN));
	m_hImageDC.SetDCPenColor(m_rPenColor);

	y1 = m_iHeight-1-y1;
	y2 = m_iHeight-1-y2;
	m_hImageDC.Rectangle(x1, y1, x2, y2);
}

void Plotter::ellipse(int x1, int y1, int x2, int y2)
{
	if (!m_hImageDC.m_hDC)
		return;

	/* set brush color */
	m_hImageDC.SelectObject(GetStockObject(DC_BRUSH));
	m_hImageDC.SetDCBrushColor(m_rFillColor);

	/* set pen color */
	m_hImageDC.SelectObject(GetStockObject(DC_PEN));
	m_hImageDC.SetDCPenColor(m_rPenColor);

	y1 = m_iHeight-1-y1;
	y2 = m_iHeight-1-y2;
	m_hImageDC.Ellipse(x1, y1, x2, y2);
}

void Plotter::alabel(char x_align, char y_align, char *str)
{
	CPoint pt;
	CSize sz;

	if (!m_hImageDC.m_hDC)
		return;

	/* move to (0, 0) and get previous position */
	pt = m_hImageDC.MoveTo(0, 0);

	/* move back to position */
	m_hImageDC.MoveTo(pt.x, pt.y);

	/* determine width and height of text */
	sz = m_hImageDC.GetTextExtent(str, (int)strlen(str));

	/* determine x offset */
	switch (x_align)
	{
	case 'l':	/* left align */
		pt.x -= sz.cx;
		break;

	case 'c':	/* center align */
		pt.x -= (sz.cx/2);
		break;

	case 'r':	/* right align */
		break;

	default:
		break;
	}

	/* determine y offset */
	switch (y_align)
	{
	case 'b':	/* bottom */
		pt.y += sz.cy;
		break;

	case 'x':	/* baseline */
		break;

	case 'c':	/* center */
		pt.y += (sz.cy/2);
		break;

	case 't':	/* top */
		pt.y -= sz.cy;
		break;

	default:
		break;
	}

	/* draw text */
	m_hImageDC.ExtTextOut(pt.x, pt.y, ETO_CLIPPED, NULL, str, 
						  (UINT)strlen(str), NULL);
}

void Plotter::parampl(char *key, char *val)
{
	if (!strcmp(key, "BITMAPSIZE"))
	{
		int r, w, h;

		r = sscanf(val, "%dx%d", &w, &h);
		if (r == 2)
		{
			m_iWidth = w;
			m_iHeight = h;
		}
	}

	m_mParams.insert(std::make_pair(std::string(key), std::string(val)));
}

void Plotter::DoMyBidding()
{
	int r;

	if (!m_hImageDC.m_hDC)
		return;

	r = m_hImageDC.SetMapMode(MM_ISOTROPIC);

	/* THIS IS IMPORTANT: x and y signs are NOT reversed from the GDI example */
	m_hImageDC.SetViewportOrg(340, 220);

	/* set pen color */
	m_hImageDC.SelectObject(GetStockObject(DC_PEN));
	m_hImageDC.SetDCPenColor(RGB(255, 0, 0));

	/* set brush color */
	m_hImageDC.SelectObject(GetStockObject(DC_BRUSH));
	m_hImageDC.SetDCBrushColor(RGB(0, 0, 0x80));

	/* draw rectangle */
	m_hImageDC.Rectangle(-100, -100, 100, 100);

	/* set pen color */
	m_hImageDC.SelectObject(GetStockObject(DC_PEN));
	m_hImageDC.SetDCPenColor(RGB(0, 0, 255));

	/* draw 45deg line */
	m_hImageDC.MoveTo(0, 0);
	m_hImageDC.LineTo(200, 200);
}

void Plotter::TestPattern_1200_2400()
{
	int i;

	pencolor(0, 0, 0);
	fillcolor(255, 0, 0);

	for (i = 0; i < 1201; i += 10)
	{
		ellipse(600, i, 605, i+5);
	}

	fillcolor(0, 255, 0);

	for (i = 0; i < 1201; i += 10)
	{
		int x;

		x = 1201+i;
		ellipse(610, x, 615, x+5);
	}
}

bool Plotter::WriteStream()
{
	BITMAP bm;
	BITMAPINFO bitinfo;

	ZeroMemory(&bitinfo, sizeof(BITMAPINFO));
	bitinfo.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
	bitinfo.bmiHeader.biBitCount = 0;

	if (!::GetDIBits(m_hImageDC.m_hDC, (HBITMAP)m_hBitmap.m_hObject, 0, 0, NULL, &bitinfo, DIB_RGB_COLORS))
		return false;

	bm.bmHeight = bitinfo.bmiHeader.biHeight;
	bm.bmWidth = bitinfo.bmiHeader.biWidth;

	BITMAPFILEHEADER bmhdr;

	bmhdr.bfType = 0x4d42;
	bmhdr.bfSize = (((3*bm.bmWidth+3)&~3)*bm.bmHeight)
				   +sizeof(BITMAPFILEHEADER)
				   +sizeof(BITMAPINFOHEADER);
	bmhdr.bfReserved1 = 0;
	bmhdr.bfReserved2 = 0;
	bmhdr.bfOffBits = (DWORD)sizeof(BITMAPFILEHEADER)+sizeof(BITMAPINFOHEADER);

	bitinfo.bmiHeader.biCompression = 0;
	fwrite(&bmhdr, sizeof(BITMAPFILEHEADER), 1, m_pFile);
	fwrite(&bitinfo.bmiHeader, sizeof(BITMAPINFOHEADER), 1, m_pFile);

	BYTE *pData = new BYTE[bitinfo.bmiHeader.biSizeImage+5];
	if (!::GetDIBits(m_hImageDC.m_hDC, (HBITMAP)m_hBitmap.m_hObject, 0, bm.bmHeight, pData, &bitinfo, DIB_RGB_COLORS))
		return false;
	if (pData != NULL)
		fwrite(pData, 1, bitinfo.bmiHeader.biSizeImage, m_pFile);

	delete [] pData;
	return true;
}

