#ifndef _threadbase_h
#define _threadbase_h

#include <process.h>
#include <exception>

class ThreadBase
{
public:
	ThreadBase(bool issuspended = false) :
			   m_pHandle(NULL),
			   m_uiId(0),
			   m_bTerminated(false),
			   m_bAmTerminated(false),
			   m_bIsSuspended(issuspended)
	{
		unsigned int flags;

		/* build flags for creating the thread */
		if (m_bIsSuspended)
			flags = CREATE_SUSPENDED;
		else flags = 0;

		/* start the thread */
		m_pHandle = (HANDLE)_beginthreadex(NULL, 0, privExecute, this, flags, &m_uiId);
		
		/* check for errors */
		if (m_pHandle == NULL)
		{
			throw new exception("_beginthreadex() failed");
		}
	}

	~ThreadBase()
	{
		/* check if the thread is running */
		if (m_pHandle)
		{
			/* terminate thread */
			m_bTerminated = true;

			/* wait for thread to terminate */
			WaitForSingleObject(m_pHandle, 1000);

			/* close thread handle */
			CloseHandle(m_pHandle);
			m_pHandle = NULL;

			/* set terminated flag */
			m_bAmTerminated = true;
		}
	}

	void Resume()
	{
		/* resume the current thread */
		ResumeThread(m_pHandle);
	}

	void Terminate()
	{
		/* set terminated flag */
		m_bTerminated = true;
	}

	void Suspend()
	{
		/* suspend the current thread */
		SuspendThread(m_pHandle);
	}

	virtual void Execute() = 0;


	inline bool Terminated() const			{ return m_bAmTerminated; }

protected:
	static unsigned int __stdcall privExecute(void *param)
	{
		ThreadBase *me;

		/* cast param to a pointer of me */
		me = (ThreadBase*)param;

		/* call member function */
		me->Execute();

		/* return OK */
		return 0;
	}

	HANDLE m_pHandle;
	unsigned int m_uiId;
	bool m_bTerminated;
	bool m_bAmTerminated;
	bool m_bIsSuspended;
};

#endif